import{_ as i,c as e,o as a,ag as t}from"./chunks/framework.jCcxW76o.js";const c=JSON.parse('{"title":"Data Formats","description":"","frontmatter":{"legacyLink":"Data.html","outline":[2,3]},"headers":[],"relativePath":"development/Data.md","filePath":"development/Data.md"}'),n={name:"development/Data.md"};function l(p,s,h,d,o,r){return a(),e("div",null,s[0]||(s[0]=[t(`<h1 id="data-formats" tabindex="-1">Data Formats <a class="header-anchor" href="#data-formats" aria-label="Permalink to &quot;Data Formats&quot;">​</a></h1><p>The CommuniGate Pro Server processes most types of data as generic <em>objects</em>. This section describes the generic, fundamental object types used in all Server components.</p><p>For each object type, a textual representation of the object data is specified.</p><p>When an object is stored in a file, sent in a <a href="./CLI.html">CLI/API</a> command response, or extracted from the Server in any other way, the object textual representation is used.</p><p>When the Server reads an object from outside (from a settings file, from a CLI command, etc.), it converts the provided textual representation into an internal object.</p><h2 id="Syntax" tabindex="-1">Textual Representation <a class="header-anchor" href="#Syntax" aria-label="Permalink to &quot;Textual Representation {#Syntax}&quot;">​</a></h2><p>Objects can be represented as a single-line or multi-line text (depending on the context).</p><p>Object representation elements can be separated using &quot;white spaces&quot;. Only the space symbol can be used as a &quot;white space&quot; in single-line texts.<br> In multi-line texts the space and &quot;horizontal tab&quot; symbols, as well as the EOL (End-Of-Line, CR and/or LF) symbols are treated as &quot;white spaces&quot;. Additionally, a &quot;comment&quot; is treated as a &quot;white space&quot;, where a comment consists of:</p><ul><li>two consecutive forward slash symbols (<code>//</code>) and all following symbols up to and including the first EOL symbol.</li><li>the consecutive forward slash and asterisk symbols (<code>/*</code>) and all following symbols up to and including the consecutive asterisk and forward slash symbols (<code>*/</code>).</li></ul><h2 id="Atomic" tabindex="-1">Atomic Objects <a class="header-anchor" href="#Atomic" aria-label="Permalink to &quot;Atomic Objects {#Atomic}&quot;">​</a></h2><p>Atomic objects are unstructured, generic data-type objects.</p><h3 id="String" tabindex="-1">Strings <a class="header-anchor" href="#String" aria-label="Permalink to &quot;Strings {#String}&quot;">​</a></h3><p>A string is a sequence of UTF-8 encoded text bytes, not containing a binary zero byte.</p><p>A textual representation of a string is either an <em>atom</em> - a sequence of Latin letters (in the ASCII encoding) and digits, or a <em>quoted string</em> - a sequence of any printable symbols (using the UTF-8 encoding) except the quotation mark and the backslash symbol, enclosed into the quotation marks (<code>&quot;</code>).</p><p>Examples:</p><ul><li><code>MyName</code></li><li><code>My2ndName</code></li><li><code>&quot;My Name with spaces and the . symbol&quot;</code></li></ul><p>If you want to include the quotation mark symbol into a quoted string, include the backslash symbol and the quotation mark, if you want to include the backslash symbol into a quoted string, include two backslash symbols.</p><p>Examples:</p><ul><li><code>&quot;a \\&quot;string\\&quot; within string&quot;</code></li><li><code>&quot;Single \\\\ backslash&quot;</code></li></ul><p>You can use the <code>\\r</code> symbol combination to include the <em>carriage-return</em> symbol into a string, you can use the <code>\\n</code> symbol combination to include the <em>line-feed</em> symbol into a string, and you can use the <code>\\e</code> symbol combination to include the system-independent <em>End-Of-Line</em> symbol(s) into a string.</p><p>Examples:</p><ul><li><code>&quot;Line1\\eLine2&quot;</code></li><li><code>&quot;TEXT3\\rTEXT67\\nTEXT78&quot;</code></li></ul><p>Use the <code>\\r</code> or <code>\\n</code> combinations to include the carriage-return and line-feed characters only when they are NOT used as line separators.</p><p>You can use the <code>\\t</code> symbol combination to include the <em>tabulation</em> symbol into a string.</p><p>Example:</p><ul><li><code>&quot;Line1:\\tField1\\tField2\\eLine2:\\tField1\\tField2&quot;</code></li></ul><p>You can use the <code>\\nnn</code> symbol combination to include any symbol into a string, if <em>nnn</em> is a 3-digit decimal number equal to the ASCII code of the desired symbol.</p><p>Example:</p><ul><li><code>&quot;Using the \\012 (Vertical Tabulation) symbol&quot;</code></li></ul><p>You can use the <code>\\u&#39;nnn&#39;</code> symbol combination to include any Unicode symbol into a string, if <em>nnn</em> is a hexadecimal number equal to the Unicode code of the desired symbol.</p><p>Example:</p><ul><li><code>&quot;Using the \\u&#39;2764&#39; (Heavy black heart) symbol&quot;</code></li></ul><p>In multi-line texts, a string can be represented as two or more consecutive strings with zero or more white spaces between them. This feature allows you to break very long string objects into several strings and put them on multiple text lines.</p><h3 id="DataBlock" tabindex="-1">DataBlocks <a class="header-anchor" href="#DataBlock" aria-label="Permalink to &quot;DataBlocks {#DataBlock}&quot;">​</a></h3><p>DataBlocks are basic, unstructured blocks of binary data. They are composed as text strings with the Base64 encoding of binary data enclosed into brackets.</p><p>Example:</p><ul><li><code>[HcqHfHI=]</code> this is a binary block containing the following 5 binary data bytes: <code>0x1D 0xCA 0x87 0x7C 0x72</code></li></ul><p>White spaces can be placed before and after the brackets and between any symbols of the encoded data.</p><h3 id="Number" tabindex="-1">Numbers <a class="header-anchor" href="#Number" aria-label="Permalink to &quot;Numbers {#Number}&quot;">​</a></h3><p>Numbers are basic, unstructured data objects. Each Number object contains one 64-bit signed integer value. A Number is presented as a text string starting with the # symbol, followed by an optional minus (<code>-</code>) symbol, followed by either 1 or more decimal digits, or by the <em>base-indicator</em> and 1 or more digits of the specified base.<br> The base indicator 0x specifies the base value 16 (a hexadecimal number), the base indicator 0o specifies the base value 8 (a octal number), the base indicator 0b specifies the base value 2 (a binary number).</p><p>Examples:</p><ul><li><code>#-234657</code></li><li><code>#0x17EF</code></li><li><code>#-0b1000111000</code></li></ul><h3 id="Date" tabindex="-1">Time Stamps <a class="header-anchor" href="#Date" aria-label="Permalink to &quot;Time Stamps {#Date}&quot;">​</a></h3><p>Time Stamps are basic, unstructured data objects. Each Time Stamp object contains one global time value. The time value is presented in GMT time, as a text string starting with the #T symbols, and containing the day, months, year, and, optionally, the hour, minute, and the second values.</p><p>Example:</p><ul><li><code>#T22-10-2009_15:24:45</code></li></ul><p>There are 2 special Time Stamps - for the &quot;remote past&quot; (<code>#TPAST</code>) and for the &quot;remote future&quot; (<code>#TFUTURE</code>).</p><h3 id="IPAddr" tabindex="-1">IP Addresses <a class="header-anchor" href="#IPAddr" aria-label="Permalink to &quot;IP Addresses {#IPAddr}&quot;">​</a></h3><p>IP Addresses are basic, unstructured data objects. Each IP Address object contains an IPv4 or IPv6 address, and, optionally a port number. The IP Address is presented as a text string starting with the #I symbols, and containing the canonical IPv4 or IPv6 address, optionally followed by the port number.</p><p>Examples:</p><ul><li><code>#I[10.0.44.55]:25</code></li><li><code>#I[2001:470:1f01:2565::a:80f]:25</code></li></ul><h3 id="Null" tabindex="-1">Null Object <a class="header-anchor" href="#Null" aria-label="Permalink to &quot;Null Object {#Null}&quot;">​</a></h3><p>A null-object is a unique atomic object used to represent absence of any other object.<br> The Null Object is presented as the #NULL# text string.</p><h3 id="Unknown" tabindex="-1">Other Objects <a class="header-anchor" href="#Unknown" aria-label="Permalink to &quot;Other Objects {#Unknown}&quot;">​</a></h3><p>When the Server converts internal data objects into the text presentation, it may encounter some custom, application-specific objects. These objects are presented as #(<em>objectName</em>:<em>address</em>) text strings, where <em>objectName</em> is an alpha-numeric object class name, and <em>address</em> is a hexadecimal number - the object memory address.</p><p>These objects must NOT be used in text presentation being converted into a data object.</p><h2 id="Structured" tabindex="-1">Structured Objects <a class="header-anchor" href="#Structured" aria-label="Permalink to &quot;Structured Objects {#Structured}&quot;">​</a></h2><h3 id="Array" tabindex="-1">Arrays <a class="header-anchor" href="#Array" aria-label="Permalink to &quot;Arrays {#Array}&quot;">​</a></h3><p>An array object is an ordered set of objects (array elements).</p><p>Array textual representation is a list of its element representations, separated with the comma (,) symbols, and enclosed into the parentheses.</p><p>Example:</p><ul><li><code>(Element1 , &quot;Element2&quot; , &quot;Element 3&quot;)</code></li></ul><p>An array element can be any object - a string, an array, a dictionary, etc.</p><p>Example:</p><ul><li><code>(Element1 , (&quot;Sub Element1&quot;, SubElement2) , &quot;Element 3&quot;)</code></li></ul><p>Any number of spaces, tabulation symbols, and/or line breaks (end-of-line symbols) can be placed between a parenthesis and an element, and between an element and a comma symbol.</p><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>  (</span></span>
<span class="line"><span>    Element1  ,</span></span>
<span class="line"><span>    (    &quot;Sub Element1&quot;,</span></span>
<span class="line"><span>     SubElement2  )</span></span>
<span class="line"><span>    ,</span></span>
<span class="line"><span>  &quot;Element 3&quot;  )</span></span></code></pre></div><p>An array may have zero elements (an empty array).</p><p>Example:</p><ul><li><code>()</code></li></ul><h3 id="Dictionary" tabindex="-1">Dictionaries <a class="header-anchor" href="#Dictionary" aria-label="Permalink to &quot;Dictionaries {#Dictionary}&quot;">​</a></h3><p>A dictionary object is a set of key-value pairs. Dictionary keys are strings. Each key in a dictionary should be unique. The dictionary keys are processed as <strong>case-sensitive</strong> strings, unless explicitly specified otherwise.</p><p>Any object can be used as a value associated with a key.</p><p>A dictionary textual representation is a sequence of its key value pairs, enclosed into the curvy brackets.<br> Each pair is represented as its key string representation, followed by the equal (<code>=</code>) symbol, followed by the textual representation of the associated value object, followed by the semicolon (<code>;</code>) symbol.</p><p>Example:</p><ul><li><code>{Key1=Element1; Key2 =&quot;Element2&quot; ; &quot;Third Key&quot;=&quot;Element 3&quot;; }</code></li></ul><p>The value object in any key-value pair can be a string, an array, a dictionary, or any other object.</p><p>Example:</p><ul><li><code>{Key1=(Elem1,Elem2); Key2={Sub1=&quot;XXX 1&quot;; Sub2=X245;}; }</code></li></ul><p>Any number of spaces, tabulation symbols, and/or line breaks (end-of-line symbols) can be placed between a bracket and a pair, around the equal symbol, and around the semicolon symbol.</p><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>  {</span></span>
<span class="line"><span>   Key1  =   (Elem1,Elem2)   ;</span></span>
<span class="line"><span>   Key2 = {  Sub1 = &quot;XXX 1&quot;;</span></span>
<span class="line"><span>      Sub2=X245;  };</span></span>
<span class="line"><span>  }</span></span></code></pre></div><p>A dictionary may have zero elements (an empty dictionary).</p><p>Example:</p><ul><li><code>{}</code></li></ul><h3 id="XML" tabindex="-1">XML <a class="header-anchor" href="#XML" aria-label="Permalink to &quot;XML {#XML}&quot;">​</a></h3><p>An XML object is an XML document. It has a name, a set of namespaces (strings), a set attributes with string values, and zero, one, or several body elements. Each body element is either a string or an XML object.</p><p>An XML object textual representation is its standard textual representation, starting with the angle bracket symbol.</p><h2 id="Rules" tabindex="-1">Formal Syntax Rules <a class="header-anchor" href="#Rules" aria-label="Permalink to &quot;Formal Syntax Rules {#Rules}&quot;">​</a></h2><p>Below is the formal syntax definition for textual representations of the basic type objects.</p><table tabindex="0"><thead><tr><th></th><th></th></tr></thead><tbody><tr><td><em>b-digit</em></td><td><code>0</code> | <code>1</code></td></tr><tr><td><em>o-digit</em></td><td><code>0</code> .. <code>7</code></td></tr><tr><td><em>d-digit</em></td><td><code>0</code> .. <code>9</code></td></tr><tr><td><em>h-digit</em></td><td><em>d-digit</em> | <code>A</code> .. <code>F</code> | <code>a</code> .. <code>f</code></td></tr><tr><td><em>a-symbol</em></td><td><code>A</code> .. <code>Z</code> | <code>a</code> .. <code>z</code> | <em>d-digit</em></td></tr><tr><td><em>u-symbol</em></td><td><em>a-symbol</em> | <em>any non-ASCII utf-8 symbol</em></td></tr><tr><td><em>l-symbol</em></td><td><em>u-symbol</em> | <code>.</code> | <code>-</code> | <code>_</code> | <code>@</code></td></tr><tr><td><em>atom</em></td><td><strong>1*</strong> <em>l-symbol</em></td></tr><tr><td><em>b-symbol</em></td><td><em>a-symbol</em> | <code>+</code> | <code>/</code> | <code>=</code></td></tr><tr><td><em>s-symbol</em></td><td><em>any printable symbol except <code>&quot;</code> and <code>\\</code></em> | <code>\\\\</code> | <code>\\&quot;</code> | <code>\\r</code> | <code>\\n</code> | <code>\\e</code> | <code>\\t</code> | <code>\\</code> <em>d-digit</em> <em>d-digit</em> <em>d-digit</em></td></tr><tr><td><em>string</em></td><td><code>&quot;</code> <strong>0*</strong> <em>s-symbol</em> <code>&quot;</code> | <em>atom</em></td></tr><tr><td><em>datablock</em></td><td><code>[</code> <strong>1*</strong> <em>b-symbol</em> <code>]</code></td></tr><tr><td><em>day</em></td><td><code>0</code> .. <code>3</code> <em>d-digit</em> <em>(2-digit number in the 1..31 range)</em></td></tr><tr><td><em>month</em></td><td><code>0</code> .. <code>1</code> <em>d-digit</em> <em>(2-digit number in the 1..12 range)</em></td></tr><tr><td><em>year</em></td><td><code>1</code> .. <code>2</code> <em>d-digit</em> <em>d-digit</em> <em>d-digit</em> <em>(4-digit number in the 1970..2038 range)</em></td></tr><tr><td><em>hour</em></td><td><code>0</code> .. <code>2</code> <em>d-digit</em> <em>(2-digit number in the 0..23 range)</em></td></tr><tr><td><em>minute</em></td><td><code>0</code> .. <code>5</code> <em>d-digit</em> <em>(2-digit number in the 0..59 range)</em></td></tr><tr><td><em>second</em></td><td><code>0</code> .. <code>5</code> <em>d-digit</em> <em>(2-digit number in the 0..59 range)</em></td></tr><tr><td><em>number</em></td><td><code>#</code> [ <code>-</code> ] <strong>1*</strong> <em>d-digit</em> | <code>#</code> [<code>-</code>] <code>0x</code> 1* <em>h-digit</em> | <code>#</code> [<code>-</code>] <code>0b</code> 1* <em>b-digit</em> | <code>#</code> [<code>-</code>] <code>0o</code> 1* <em>o-digit</em></td></tr><tr><td><em>timestamp</em></td><td><code>#T</code> <em>day</em> <code>-</code> <em>month</em> <code>-</code> <em>year</em> [ <code>_</code> <em>hour</em> <code>:</code> <em>minute</em> <code>:</code> <em>second</em> ]</td></tr><tr><td><em>ip4</em></td><td><strong>1*</strong> <em>d-digit</em> <code>.</code> <strong>1*</strong> <em>d-digit</em> <code>.</code> <strong>1*</strong> <em>d-digit</em> <code>.</code> <strong>1*</strong> <em>d-digit</em></td></tr><tr><td><em>ip6</em></td><td><strong>0*</strong> (<strong>1*</strong> <em>h-digit</em> <code>:</code>) [<code>:</code> [ 0*(<strong>1*</strong> <em>h-digit</em> <code>:</code>) ] ]</td></tr><tr><td><em>ip-address</em></td><td><code>#I[</code> [ <em>ip4</em> | <em>ip6</em> ] <code>]</code> [ <code>:</code> <strong>1*</strong> <em>d-digit</em> ]</td></tr><tr><td><em>null-object</em></td><td><code>#NULL#</code></td></tr><tr><td><em>array</em></td><td><code>(</code> <code>[</code> <em>object</em> <strong>0*</strong> ( <code>,</code> <em>object</em> ) <code>]</code> <code>)</code></td></tr><tr><td><em>dictionary</em></td><td><code>{</code> <strong>0*</strong> ( <em>string</em> <code>=</code> <em>object</em> <code>;</code> ) <code>}</code></td></tr><tr><td><em>XML</em></td><td><code>&lt;</code> <em>XML standard format</em> <code>\\&gt;</code></td></tr><tr><td><em>object</em></td><td><em>string</em> | <em>datablock</em> | <em>number</em> | <em>timestamp</em> | <em>ip-address</em> | <em>null-object</em> | <em>array</em> | <em>dictionary</em> | <em>XML</em></td></tr></tbody></table><h2 id="XMLObjects" tabindex="-1">XML Objects <a class="header-anchor" href="#XMLObjects" aria-label="Permalink to &quot;XML Objects {#XMLObjects}&quot;">​</a></h2><p>CommuniGate Pro can convert complex structures (such as vCards, iCalendar, SDP objects) into generic XML objects. An XML presentation may also be required for Objects such as datablocks, arrays, or dictionaries. This section specifies the CommuniGate Pro XML presentation for all these objects and complex structures.</p><h3 id="XMLString" tabindex="-1">String <a class="header-anchor" href="#XMLString" aria-label="Permalink to &quot;String {#XMLString}&quot;">​</a></h3><p>A string is presented as a text element.<br> If this is a top XML element, a string is presented as an <code>object</code> XML element, with the text body containing this string.</p><p>If a string contains special characters, it should be represented as a <code>binString</code> element with a text body containing the base64-encoded string.<br> An empty string is presented as an empty <code>binString</code> element.</p><h3 id="XMLDataBlock" tabindex="-1">Datablock <a class="header-anchor" href="#XMLDataBlock" aria-label="Permalink to &quot;Datablock {#XMLDataBlock}&quot;">​</a></h3><p>A datablock is presented as a <code>base64</code> XML element.<br> The XML element body is the base64-encoded datablock content.</p><p>Example:</p><ul><li><code>&lt;base64&gt;STYRyui=&lt;/base64&gt;</code></li></ul><h3 id="XMLNumber" tabindex="-1">Number <a class="header-anchor" href="#XMLNumber" aria-label="Permalink to &quot;Number {#XMLNumber}&quot;">​</a></h3><p>A number is presented as a <code>number</code> XML element.<br> The XML element body is the text presentation of the number object value (a decimal value, or, if the base indicator is given, a hexadecimal, octal, or binary value).</p><p>Examples:</p><ul><li><code>&lt;number&gt;123456&lt;/number&gt;, &lt;number&gt;0x78FAB5&lt;/number&gt;</code></li></ul><h3 id="XMLDate" tabindex="-1">Time Stamp <a class="header-anchor" href="#XMLDate" aria-label="Permalink to &quot;Time Stamp {#XMLDate}&quot;">​</a></h3><p>A Time Stamp is presented as a <code>date</code> XML element.<br> The XML element body is the text presentation of the time stamp in the iCalendar format.</p><p>Example:</p><ul><li><code>&lt;date&gt;20101122T123000Z&lt;/date&gt;</code></li></ul><h3 id="XMLIPAddr" tabindex="-1">IP Addresses <a class="header-anchor" href="#XMLIPAddr" aria-label="Permalink to &quot;IP Addresses {#XMLIPAddr}&quot;">​</a></h3><p>An IP Address is presented as an <code>ipAddr</code> XML element.<br> The XML element body is the text presentation of the IP address, enclosed with square brackets, and optional decimal port number separated with the colon symbol.</p><p>Example:</p><ul><li><code>&lt;ipAddr&gt;[10.0.2.2]:8010&lt;/ipAddr&gt;</code></li></ul><h3 id="XMLNull" tabindex="-1">Null-value <a class="header-anchor" href="#XMLNull" aria-label="Permalink to &quot;Null-value {#XMLNull}&quot;">​</a></h3><p>A null-value object is presented as an empty <code>null</code> XML element.</p><h3 id="XMLArray" tabindex="-1">Array <a class="header-anchor" href="#XMLArray" aria-label="Permalink to &quot;Array {#XMLArray}&quot;">​</a></h3><p>An <a href="#Array">array</a> is presented as a sequence of one or more <code>subValue</code> XML elements.<br> The XML element body represents an Array element.<br> An empty array is presented as one <code>subValue</code> XML element without a body.<br> If this is a top XML element, the array is presented as an <code>object</code> XML element, with the text body containing <code>subValue</code> XML elements.</p><p>Example:</p><ul><li><code>&lt;subValue&gt;my string&lt;/subValue&gt;&lt;subValue&gt;&lt;number&gt;123456&lt;/number&gt;&lt;/subValue&gt;</code></li></ul><h3 id="XMLDictionary" tabindex="-1">Dictionary <a class="header-anchor" href="#XMLDictionary" aria-label="Permalink to &quot;Dictionary {#XMLDictionary}&quot;">​</a></h3><p>A <a href="#Dictionary">dictionary</a> is presented as a sequence of one or more <code>subKey</code> XML elements.<br> The XML element <code>key</code> attribute presents the dictionary element key, and the XML body represents the dictionary element value.<br> An empty dictionary is presented as one <code>subKey</code> XML element without a <code>key</code> attribute and without a body.<br> If this is a top XML element, the dictionary is presented as an <code>object</code> XML element, with its body containing <code>subKey</code> XML elements.</p><p>Example:</p><ul><li><code>&lt;subKey key=&quot;firstKey&quot;&gt;my string&lt;/subKey&gt;&lt;subKey key=&quot;secondKey&quot;&gt;&lt;number&gt;123456&lt;/number&gt;&lt;/subKey&gt;</code></li></ul><h3 id="XMLvCard" tabindex="-1"><code>vCard</code> <a class="header-anchor" href="#XMLvCard" aria-label="Permalink to &quot;\`vCard\` {#XMLvCard}&quot;">​</a></h3><p>This XML element represents a vCard object (as specified in the Jabber/XEP vCard XML documents).</p><ul><li><p>Attributes:</p><ul><li><code>modified</code></li></ul><p>This optional attribute contains the value of the <code>REV</code> property (iCalendar-formatted GMT time).</p></li><li><p>Body:</p></li></ul><p>Contains vCard properties as XML elements with the same names, converted to the uppercase ASCII.<br> Each property element contains:</p><ul><li><p>Attributes:</p><p><em>none</em></p></li><li><p>Body:</p><p>a set of XML sub-elements representing property parameters and the property value:</p><ul><li><p>parameters</p><p>XML elements with the same names as the parameter names, converted to the uppercase ASCII. The <code>ENCODING</code> and <code>QUOTED-PRINTABLE</code> vCard property parameters are used to decode the property value and they are not stored.</p></li><li><p>value</p><ul><li><p><em>structured value</em> (<code>N</code>,<code>ORG</code>,<code>ADR</code>)</p><p>a set of XML elements with structure element names, and text bodies containing a subpart of the structured property value.</p></li><li><p><em>binary value</em></p><p>a <code>BINVAL</code> XML element with a text body containing the base64-encoded property value.</p></li><li><p><em>text value</em></p><p>a <code>VALUE</code> XML element with a text body containing the property value.</p></li></ul></li></ul></li></ul><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>begin:VCARD</span></span>
<span class="line"><span> source:ldap://cn=bjorn Jensen, o=university of Michigan, c=US</span></span>
<span class="line"><span> name:Bjorn Jensen</span></span>
<span class="line"><span> n:Jensen;bjorn;A;Mr;II</span></span>
<span class="line"><span> email;type=INTERNET:bjorn@umich.edu</span></span>
<span class="line"><span> org:U of Michigan;Computer Science Dept.</span></span>
<span class="line"><span> tel;type=WORK,MSG:+1 313 747-4454</span></span>
<span class="line"><span> key;type=x509;encoding=B:dGhpcyBjb3VsZCBiZSAKbXkgY2VydGlmaWNhdGUK</span></span>
<span class="line"><span> end:VCARD</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vCard</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">SOURCE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;ldap://cn=bjorn Jensen, o=university of Michigan, c=US&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">SOURCE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Bjorn Jensen&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">N</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">FAMILY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Jensen&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">FAMILY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">GIVEN</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;bjorn&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">GIVEN</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MIDDLE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;A&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MIDDLE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">PREFIX</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Mr.&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">PREFIX</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">SUFFIX</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;II&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">SUFFIX</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">N</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">EMAIL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;bjorn@umich.edu&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">EMAIL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORG</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORGNAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;U of Michigan&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORGNAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORGUNIT</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Computer Science Dept.&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORGUNIT</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">ORG</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">TEL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">WORK</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MSG</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;+1 313 747-4454&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">TEL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">KEY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">X509</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BINVAL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;dGhpcyBjb3VsZCBiZSAKbXkgY2VydGlmaWNhdGUK&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BINVAL</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">KEY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vCard</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h3 id="XMLvCardGroup" tabindex="-1"><code>vCardGroup</code> <a class="header-anchor" href="#XMLvCardGroup" aria-label="Permalink to &quot;\`vCardGroup\` {#XMLvCardGroup}&quot;">​</a></h3><p>This XML element represents a vCardGroup object.</p><ul><li><p>Attributes:</p><ul><li><p><code>modified</code></p><p>This optional attribute contains the value of the <code>REV</code> property (iCalendar-formatted GMT time).</p></li><li><p>Body:</p></li></ul></li></ul><p>vCardGroup properties as XML elements with the same names, converted to the uppercase ASCII: <code>NAME</code>, <code>NOTE</code>, <code>UID</code>. Each element includes a <code>VALUE</code> XML element with a text body containing the property value.</p><p>Zero, one, or several <code>MEMBER</code> XML Elements, one for each group member. The XML element text body is the group member address, while an optional <code>CN</code> attribute can contain the element &quot;real name&quot;.</p><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>BEGIN:VGROUP</span></span>
<span class="line"><span> PRODID:CommuniGate Pro 5.2</span></span>
<span class="line"><span> VERSION:1.0</span></span>
<span class="line"><span> NAME:Basket Buddies</span></span>
<span class="line"><span> NOTE:My basketball team.</span></span>
<span class="line"><span> UID:594895837.1@team.dom</span></span>
<span class="line"><span> REV:20071214T174114Z</span></span>
<span class="line"><span> MEMBER;CN=&quot;Jack Nimble&quot;:jack@nimble.dom</span></span>
<span class="line"><span> MEMBER;CN=&quot;Tim Slow&quot;:tim@slow.dom</span></span>
<span class="line"><span> END:VGROUP</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vCardGroup</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> modified</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20071214T174114Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Basket Buddies&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NAME</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NOTE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;My basketball team.&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">NOTE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">UID</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;594895837.1@team.dom&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">VALUE</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">UID</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MEMBER</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> CN</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;Jack Nimble&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;jack@nimble.dom&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MEMBER</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MEMBER</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> CN</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;Tim Slow&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;tim@slow.dom&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">MEMBER</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vCardGroup</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h3 id="XMLiCalendar" tabindex="-1"><code>iCalendar</code> <a class="header-anchor" href="#XMLiCalendar" aria-label="Permalink to &quot;\`iCalendar\` {#XMLiCalendar}&quot;">​</a></h3><p>This XML element represents an iCalendar object.</p><ul><li><p>Body:</p><p>a <a href="#XMLvCalendar"><code>vCalendar</code></a> element.</p></li></ul><h4 id="XMLvCalendar" tabindex="-1"><code>vCalendar</code> <a class="header-anchor" href="#XMLvCalendar" aria-label="Permalink to &quot;\`vCalendar\` {#XMLvCalendar}&quot;">​</a></h4><p>This XML element represents a vCalendar object.</p><ul><li><p>Attributes:</p><ul><li><p><code>version</code></p><p>the vCalendar version (<code>2.0</code> for iCalendar)</p></li><li><p><code>method</code></p><p>an optional attribute with a vCalendar method</p></li><li><p><code>prodid</code></p><p>an optional identification string of the product used to create this vCalendar object.</p></li></ul></li><li><p>Body:</p></li></ul><p>a set of <a href="#XMLvtimezone"><code>vtimezone</code></a>, <a href="#XMLvevent"><code>vevent</code></a>, <a href="#XMLvtodo"><code>vtodo</code></a>, <a href="#XMLvfreebusy"><code>vfreebusy</code></a> elements.</p><h4 id="XMLvtimezone" tabindex="-1"><code>vtimezone</code> <a class="header-anchor" href="#XMLvtimezone" aria-label="Permalink to &quot;\`vtimezone\` {#XMLvtimezone}&quot;">​</a></h4><p>This XML element represents a VTIMEZONE object.</p><ul><li><p>Body:</p><p>a set of XML elements:</p><ul><li><code>tzid</code>: the element body contains the zone name.</li><li><code>standard</code> and, optionally, <code>daylight</code>: these element bodies contain the following XML elements: <ul><li><code>tzoffsetto</code>: the element body contains the time shift value.</li><li><code>tzoffsetfrom</code> (optional): the element body contains the time shift value.</li><li><code>rrule</code> (optional): the element body contains a string with a recurrence descriptor.</li></ul></li></ul><p>Example:</p></li></ul><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>BEGIN:VTIMEZONE</span></span>
<span class="line"><span>TZID:Europe/Central</span></span>
<span class="line"><span>BEGIN:STANDARD</span></span>
<span class="line"><span>DTSTART:19710101T030000</span></span>
<span class="line"><span>TZOFFSETFROM:+0200</span></span>
<span class="line"><span>TZOFFSETTO:+0100</span></span>
<span class="line"><span>RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU</span></span>
<span class="line"><span>END:STANDARD</span></span>
<span class="line"><span>BEGIN:DAYLIGHT</span></span>
<span class="line"><span>DTSTART:19710101T020000</span></span>
<span class="line"><span>TZOFFSETFROM:+0100</span></span>
<span class="line"><span>TZOFFSETTO:+0200</span></span>
<span class="line"><span>RRULE:FREQ=YEARLY;BYMONTH=3;BYDAY=-1SU</span></span>
<span class="line"><span>END:DAYLIGHT</span></span>
<span class="line"><span>END:VTIMEZONE</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vtimezone</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Europe/Central&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">standard</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;19710101T030000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzoffsetto</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;+0100&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzoffsetto</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">standard</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">daylight</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;19710101T020000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzoffsetto</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;+0200&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tzoffsetto</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;FREQ=YEARLY;BYMONTH=3;BYDAY=-1SU&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">daylight</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vtimezone</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h4 id="xmlvevent" tabindex="-1"><code>vevent</code>, <code>vtodo</code> <a class="header-anchor" href="#xmlvevent" aria-label="Permalink to &quot;\`vevent\`, \`vtodo\` {#xmlvevent}&quot;">​</a></h4><p>These XML elements represent VEVENT and VTODO objects.</p><ul><li><p>Attributes:</p><ul><li><p><code>localTime</code></p><p>this attribute is present for non-recurrent objects only. It contains the date and time the object starts. The time is the local time in the Time Zone selected for the current user.</p></li><li><p><code>localStart</code></p><p>this attribute is present for recurrent objects only. It contains the time the object starts (seconds since midnight). The time is the local time in the Time Zone selected for the current user.</p></li></ul></li><li><p>Body:</p><p>a set of XML elements, each representing one property.<br> The property parameters are represented as element attributes.<br> The property value is represented as the element body.<br> If the value starts with the <code>MAILTO:</code> prefix, this prefix is removed.<br> If an object has &quot;exceptions&quot;, the <code>exceptions</code> XML element is added. Its XML sub-elements represent the exception VEVENT or VTODO objects.</p></li></ul><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>BEGIN:VEVENT</span></span>
<span class="line"><span>ORGANIZER;CN=&quot;Jim Smith&quot;:MAILTO:jim\\_smith@example.com</span></span>
<span class="line"><span>RRULE:FREQ=WEEKLY;INTERVAL=2;BYDAY=MO,TU,WE,TH;UNTIL=20060305T000000Z</span></span>
<span class="line"><span>DTSTAMP:20051204T140844Z</span></span>
<span class="line"><span>UID:566852630.4@mail.example.com</span></span>
<span class="line"><span>SEQUENCE:1</span></span>
<span class="line"><span>SUMMARY:test - recurrent</span></span>
<span class="line"><span>DTSTART;TZID=NorthAmerica/Pacific:20051204T100000</span></span>
<span class="line"><span>DTEND;TZID=NorthAmerica/Pacific:20051204T110000</span></span>
<span class="line"><span>X-MICROSOFT-CDO-BUSYSTATUS:BUSY</span></span>
<span class="line"><span>LAST-MODIFIED:20051204T140844Z</span></span>
<span class="line"><span>CREATED:20051204T140844Z</span></span>
<span class="line"><span>PRIORITY:5</span></span>
<span class="line"><span>END:VEVENT</span></span>
<span class="line"><span>BEGIN:VEVENT</span></span>
<span class="line"><span>UID:566852630.4@mail.example.com</span></span>
<span class="line"><span>RECURRENCE-ID:20051206T180000Z</span></span>
<span class="line"><span>SUMMARY:test - recurrent (later this time)</span></span>
<span class="line"><span>DTSTART;TZID=NorthAmerica/Pacific:20051206T120000</span></span>
<span class="line"><span>DTEND;TZID=NorthAmerica/Pacific:20051206T130000</span></span>
<span class="line"><span>END:VEVENT</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vevent</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">organizer</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> CN</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;Jim Smith&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;jim_smith@example.com&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">organizer</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;FREQ=WEEKLY;INTERVAL=2;BYDAY=MO,TU,WE,TH;UNTIL=20060305T000000Z&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">rrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstamp</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051204T140844Z&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstamp</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">uid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;566852630.4@mail.example.com&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">uid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">sequence</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;1&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">sequence</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">summary</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;test - recurrent&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">summary</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;NorthAmerica/Pacific&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051204T100000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtend</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;NorthAmerica/Pacific&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051204T110000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">busystatus</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;BUSY&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">busystatus</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">last-modified</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051204T140844Z&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">last-modified</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">created</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051204T140844Z&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">created</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">priority</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;5&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">priority</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">exceptions</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">uid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;566852630.4@mail.example.com&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">uid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">recurrenceId</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051206T180000Z&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">recurrenceId</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">summary</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;test - recurrent (later this time)&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">summary</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;NorthAmerica/Pacific&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051206T120000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtend</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> tzid</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;NorthAmerica/Pacific&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;20051206T130000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">exceptions</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vevent</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h4 id="XMLvfreebusy" tabindex="-1"><code>vfreebusy</code> <a class="header-anchor" href="#XMLvfreebusy" aria-label="Permalink to &quot;\`vfreebusy\` {#XMLvfreebusy}&quot;">​</a></h4><p>These XML elements represent VFREEBUSY objects.</p><ul><li><p>Attributes:</p><ul><li><p><code>dtstart</code></p><p>the start time of the time period covered with this VFREEBUSY object.</p></li><li><p><code>dtend</code></p><p>the end time of the time period covered with this VFREEBUSY object.</p></li><li><p><code>dtstamp</code></p><p>the time when this VFREEBUSY object was composed.</p></li></ul></li><li><p>Body:</p><p>a set of <code>freebusy</code> XML elements, each representing one time interval. Time intervals are not intersecting.</p></li><li><p>Attributes:</p><ul><li><p><code>dtstart</code></p><p>the start time of the time interval.</p></li><li><p><code>dtend</code></p><p>the end time of the time interval.</p></li><li><p><code>fbtype</code></p><p>the optional busy-type status. If not specified, the time interval has the <code>BUSY</code> status.</p></li></ul></li></ul><p>Example:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>BEGIN:VFREEBUSY</span></span>
<span class="line"><span>DTSTART:20080329T075517Z</span></span>
<span class="line"><span>DTEND:20080604T075517Z</span></span>
<span class="line"><span>DTSTAMP:20080405T075517Z</span></span>
<span class="line"><span>FREEBUSY:20080329T075517Z/20080329T120000Z</span></span>
<span class="line"><span>FREEBUSY:20080330T070000Z/20080330T120000Z</span></span>
<span class="line"><span>FREEBUSY:20080331T070000Z/20080331T120000Z</span></span>
<span class="line"><span>END:VFREEBUSY</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vfreebusy</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080604T075517Z&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtstamp</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080405T075517Z&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080329T075517Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">freebusy</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080329T120000Z&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080329T075517Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">freebusy</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080330T120000Z&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080330T070000Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">freebusy</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtend</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080331T120000Z&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> dtstart</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20080331T070000Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">vfreebusy</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h3 id="XMLRRULE" tabindex="-1"><code>xrule</code> <a class="header-anchor" href="#XMLRRULE" aria-label="Permalink to &quot;\`xrule\` {#XMLRRULE}&quot;">​</a></h3><p>This XML element represents an recurrence object.</p><ul><li><p>Attributes:</p><ul><li><p><code>freq</code></p><p>the recurrence type.</p></li><li><p><code>interval</code></p><p>the interval parameter. If absent, interval is assumed to be 1.</p></li><li><p><code>wkst</code></p><p>the week start day name. If absent, the <code>MO</code> (Monday) is assumed.</p></li><li><p><code>count</code></p><p>an optional integer attribute specifying the COUNT parameter.</p></li><li><p><code>until</code></p><p>an optional attribute specifying the UNTIL parameter in the iCalendar date-time format.</p></li></ul></li><li><p>Body:</p></li></ul><p>a set of XML elements:</p><ul><li><p><code>BYMONTH</code>, <code>BYWEEKNO</code>, <code>BYYEARDAY</code>, <code>BYMONTHDAY</code>, <code>BYSETPOS</code></p><p>each element body contains a number - the month number, the week number, etc. Except for the <code>BYMONTH</code> element, the number can be negative.</p></li><li><p><code>BYDAY</code></p><p>each element body contains one day name (<code>MO</code>, <code>TU</code>, .. <code>SU</code>); each element can contain an optional numeric <code>week</code> attribute - the week number (it can be negative).</p></li></ul><p>Example:</p><ul><li><code>RRULE:FREQ=WEEKLY;INTERVAL=2;BYDAY=MO,TU,WE,TH;UNTIL=20060305T000000Z</code></li></ul><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">xrule</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> freq</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;WEEKLY&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> interval</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;2&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> until</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;20060305T000000Z&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;MO&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;TU&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;WE&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;TH&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">xrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><p>Example:</p><ul><li><code>RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU</code></li></ul><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">xrule</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> freq</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;YEARLY&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYMONTH</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;10&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYMONTH</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> week</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;SU&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">BYDAY</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">xrule</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h3 id="XMLCertificate" tabindex="-1"><code>x509</code> <a class="header-anchor" href="#XMLCertificate" aria-label="Permalink to &quot;\`x509\` {#XMLCertificate}&quot;">​</a></h3><p>This XML element represents an X.509 Certificate.</p><ul><li><p>Attributes:</p><ul><li><p><code>version</code></p><p>the Certificate format version.</p></li><li><p><code>subject</code></p><p>optional; an E-mail address associated with this Certificate.</p></li></ul></li><li><p>Body:</p><p>a set of XML elements:</p><ul><li><p><code>subject</code>,<code>issuer</code></p><p>these XML elements contain XML sub-elements for the name parts of the Certificate Subject and Certificate Issuer. Supported sub-elements include <code>c</code>, <code>country</code>, <code>l</code>, <code>city</code>, etc. The sub-element value is the textual body of the sub-element.</p></li><li><p><code>validFrom</code>,<code>validTill</code></p><p>a textual body of each of these elements is the valifity date-time presented in the iCalendar format</p></li><li><p><code>base64</code></p><p>an optional element - its body is the base64-encoded certificate data.</p></li></ul></li></ul><h3 id="XMLSDP" tabindex="-1"><code>sdp</code> <a class="header-anchor" href="#XMLSDP" aria-label="Permalink to &quot;\`sdp\` {#XMLSDP}&quot;">​</a></h3><p>This XML element represents an SDP object.</p><ul><li><p>Attributes:</p><ul><li><p><code>ip</code></p><p>the default media IP address (optional).</p></li><li><p><code>origUser</code></p><p>the username field of the session origin (optional).</p></li><li><p><code>sessionID</code></p><p>the sess-id field of the session origin.</p></li><li><p><code>sessionVersion</code></p><p>the sess-version field of the session origin.</p></li><li><p><code>origIP</code></p><p>the IP address specified in the nettype, addrtype, and unicast-address fields of the session origin (optional).</p></li><li><p><code>subject</code></p><p>the session subject (optional). If missing, the <code>-</code> subject is used.</p></li></ul></li><li><p>Body:</p></li></ul><p>a set of <a href="#XMLSDPInfo"><em>info</em></a>, <a href="#XMLSDPAttr"><code>attribute</code></a>, and/or <a href="#XMLSDPMedia"><code>media</code></a> elements.</p><h4 id="XMLSDPMedia" tabindex="-1"><code>media</code> <a class="header-anchor" href="#XMLSDPMedia" aria-label="Permalink to &quot;\`media\` {#XMLSDPMedia}&quot;">​</a></h4><p>This XML element represents an SDP Media object.</p><ul><li><p>Attributes:</p><ul><li><p><code>media</code></p><p>the media type (such as <code>audio</code>, <code>video</code>). If this attribute is missing, the <code>audio</code> value is used.</p></li><li><p><code>ip</code></p><p>the media address and port.</p></li><li><p><code>protocol</code></p><p>the media protocol (such as <code>udp</code>, <code>tcp</code>, <code>RTP/AVP</code>). If this attribute is missing, the <code>RTP/AVP</code> value is used.</p></li><li><p><code>direction</code></p><p>the media direction (<code>sendrecv</code>, <code>sendonly</code>, <code>recvonly</code>, <code>inactive</code>).</p></li><li><p><code>rtcp</code></p></li></ul><p>the rtcp address and port (optional)</p></li><li><p>Body:</p></li></ul><p>a set of <a href="#XMLSDPCodec"><code>codec</code></a>, <a href="#XMLSDPInfo"><em>info</em></a>, and <a href="#XMLSDPAttr"><code>attribute</code></a> elements.</p><h4 id="XMLSDPCodec" tabindex="-1"><code>codec</code> <a class="header-anchor" href="#XMLSDPCodec" aria-label="Permalink to &quot;\`codec\` {#XMLSDPCodec}&quot;">​</a></h4><p>This XML element represents an SDP Media codec.</p><ul><li><p>Attributes:</p><ul><li><p><code>id</code></p><p>the codec ID - the RTP payload number (a number in the 0..127 range).</p></li><li><p><code>name</code></p><p>the codec name (such as <code>PCMU/8000</code>).</p></li><li><p><code>format</code></p><p>the codec format parameter (optional).</p></li></ul></li></ul><h4 id="XMLSDPAttr" tabindex="-1"><code>attr</code> <a class="header-anchor" href="#XMLSDPAttr" aria-label="Permalink to &quot;\`attr\` {#XMLSDPAttr}&quot;">​</a></h4><p>This XML element represents an SDP or SDP Media attribute.</p><ul><li><p>Attributes:</p><ul><li><p><code>name</code></p><p>the attribute name.</p></li></ul></li><li><p>Body:</p></li></ul><p>optional: an attribute value string.</p><h4 id="XMLSDPInfo" tabindex="-1"><em>info</em> <a class="header-anchor" href="#XMLSDPInfo" aria-label="Permalink to &quot;_info_ {#XMLSDPInfo}&quot;">​</a></h4><p>This XML element represents an SDP or SDP Media information field element. Supported elements are:<br> for SDP: <code>info</code>, <code>uri</code>, <code>email</code>, <code>phone</code>, <code>phone</code>, <code>bandwidth</code>, <code>time</code>, <code>repeat</code>, <code>zone</code>, and <code>key</code> elements;<br> for SDP Media: <code>title</code>, <code>bandwidth</code>, and <code>key</code> elements;</p><ul><li><p>Attributes:</p><p><em>none</em>.</p></li><li><p>Body:</p></li></ul><p>optional: a field value string.<br> Example: a sample SDP document and its XML presentation:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>v=0</span></span>
<span class="line"><span>o=- 6385718 9999 IN IP4 192.168.1.65</span></span>
<span class="line"><span>s=-</span></span>
<span class="line"><span>e=support@communigatepro.ru</span></span>
<span class="line"><span>c=IN IP4 192.168.1.65</span></span>
<span class="line"><span>t=0 0</span></span>
<span class="line"><span>a=sdpattr1:sdpvalue</span></span>
<span class="line"><span>a=sdpattr2</span></span>
<span class="line"><span>m=audio 16398 RTP/AVP 0 4 8 101</span></span>
<span class="line"><span>t=title value</span></span>
<span class="line"><span>a=rtpmap:0 PCMU/8000</span></span>
<span class="line"><span>a=rtpmap:4 G723/8000</span></span>
<span class="line"><span>a=rtpmap:8 PCMA/8000</span></span>
<span class="line"><span>a=rtpmap:101 telephone-event/8000</span></span>
<span class="line"><span>a=fmtp:101 0-15</span></span>
<span class="line"><span>a=sendrecv</span></span>
<span class="line"><span>a=mediaattr1:mediavalue</span></span>
<span class="line"><span>a=mediaattr2</span></span></code></pre></div><div class="language-xml vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">xml</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">sdp</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> ip</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;[192.168.1.65]&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> origUser</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> sessionID</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;6385718&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> sessionVersion</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;9999&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> originIP</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;[192.168.1.65]&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">email</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;support@communigatepro.ru&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">email</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;sdpattr1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;sdpvalue&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;sdpattr2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">media</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> media</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;audio&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> ip</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;[192.168.1.65]:16398&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> protocol</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;RTP/AVP&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> direction</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;sendrecv&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">codec</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> id</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;PCMU/8000&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">codec</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> id</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;4&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;G723/8000&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">codec</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> id</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;8&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;PCMA/8000&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">codec</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> id</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;101&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;telephone-event/8000&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> format</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0-15&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">title</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;title value&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">title</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;mediaattr1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;mediavalue&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">attr</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> name</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;mediaattr2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> /&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">media</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">sdp</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><h2 id="ASN1" tabindex="-1">ASN.1 <a class="header-anchor" href="#ASN1" aria-label="Permalink to &quot;ASN.1 {#ASN1}&quot;">​</a></h2><p>CommuniGate Pro can convert ASN.1 structures into generic objects.</p><p>Each ASN.1 data element is converted into an <a href="#Array">array</a>.</p><p>For ASN.1 data elements of the Universal class, the array element number 0 is a string with the data element type: boolean, integer, bits, octets, null, oid, objdescr, external, real, enum, embed, string, reloid, rsrv1, rsrv2, seq, set, stringNum, stringPrint, stringT61, stringIA5, string22, timeUTC, time, stringGraphic, stringISO64, stringGeneral, stringUniversal, string29, stringBMP, string31.<br> Otherwise the array element number 0 is a string with the data element class (application, choice,private), and the array element number 1 is a number - the data element type code.</p><p>For &quot;constructed&quot; data elements, the remaining zero or more array elements contain converted ASN.1 data sub-elements.<br> For &quot;primitive&quot; data elements, the remaining array element contain the the data element presentation, which depends on the data element type:</p><ul><li><p><code>boolean</code></p><p>a <a href="#Null">Null object</a> for &quot;false&quot;, any other object for &quot;true&quot;</p></li><li><p><code>integer</code>, <code>enum</code> (not more than 8 bytes long)</p><p>a <a href="#Number">Number</a></p></li><li><p><code>null</code></p><p>a <a href="#Null">Null object</a></p></li><li><p><code>string</code>, <code>subtypestring</code> (the data does not contain a binary zero)</p><p>a <a href="#String">String</a></p></li><li><p><code>time</code>, <code>timeUTC</code></p><p>a <a href="#Date">Time Stamp</a></p></li><li><p><code>oid</code>, <code>reloid</code></p><p>a <a href="#String">String</a> representing the ObjectID (<em>nn</em>.<em>mm</em>.<em>ll</em>.<em>kk</em>.<em>rr</em>)</p></li><li><p><em>all other data types</em></p><p>a <a href="#DataBlock">Datablock</a></p></li></ul>`,204)]))}const E=i(n,[["render",l]]);export{c as __pageData,E as default};
