import{_ as t,c as s,o as i,ag as a}from"./chunks/framework.jCcxW76o.js";const u=JSON.parse('{"title":"Web Server-Side Programming (WSSP)","description":"","frontmatter":{"legacyLink":"WSSP.html"},"headers":[],"relativePath":"development/WSSP.md","filePath":"development/WSSP.md"}'),n={name:"development/WSSP.md"};function o(l,e,h,r,d,p){return i(),s("div",null,e[0]||(e[0]=[a(`<h1 id="web-server-side-programming-wssp" tabindex="-1">Web Server-Side Programming (WSSP) <a class="header-anchor" href="#web-server-side-programming-wssp" aria-label="Permalink to &quot;Web Server-Side Programming (WSSP)&quot;">​</a></h1><p>The CommuniGate Pro Web Application module processes a request for a WSSP file by calling a <a href="./WebApp.html#Processing">code component</a> that produces a <em>dataset</em> - a <a href="./Data.html">dictionary</a> containing text string keys and values, associated with those keys. Values can be text strings, numbers, timestamp, arrays, or dictionaries. See the <a href="./Data.html">Data</a> section for more details on data types.</p><p>For example, when a Domain default page is requested, the code component is called. The component processes request (HTML FORM) parameters and produces a <em>dataset</em> - a <em>dictionary</em> containing keyed values. For example, a dataset produced with the component processing the Login requests may contain <code>canAutoSignup</code>, <code>hasMailLists</code>, and <code>hasCertificate</code> keys.</p><p>The Web Application module then uses the script code from a WSSP file to convert this dataset into a markup language (HTML, WML, etc.) page.</p><h2 id="Elements" tabindex="-1">Scripting Elements <a class="header-anchor" href="#Elements" aria-label="Permalink to &quot;Scripting Elements {#Elements}&quot;">​</a></h2><p>The WSSP file is a markup language (usually - HTML) file with two additional types of elements:</p><ul><li>text elements, started and ended with double percent (<code>%%</code>) symbols</li><li>structural elements, started with the <code>&lt;!--%%</code> marker and ended with the <code>--&gt;</code> marker.</li></ul><p>The following is a sample of an WSSP document:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">html</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">body</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">h1</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Welcome to %%server%%. Your ID is %%ID%%.&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">h1</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">    &lt;!--%%IF EXISTS(lastLogin)--&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    Last time you visited us on %%lastLogin%%</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">    &lt;!--%%ENDIF--&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">body</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">html</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><p>This WSSP document contains the <code>%%server%%</code>, <code>%%ID%%</code>, and <code>%%lastLogin%%</code> text elements, and the <code>&lt;!--%%IF EXISTS(lastLogin)--&gt;</code> and <code>&lt;!--%%ENDIF--&gt;</code> structural elements (these text elements are fictitious, do not try to use these samples in your real .wssp pages).</p><p>If the WSSP document should contain non-ASCII symbols, the UTF-8 character set should be used. When the WSSP document is being processed, the Web Application module retrieves the <code>charset</code> string value from the produced data dictionary. If this value is not <code>UTF-8</code>, then the WSSP text is converted into this <em>page charset</em>.</p><h2 id="Exprs" tabindex="-1">Expressions <a class="header-anchor" href="#Exprs" aria-label="Permalink to &quot;Expressions {#Exprs}&quot;">​</a></h2><p>The text and structural WSSP elements use expressions - combinations of names and symbols that specify the data to be retrieved from the data dictionary or from other available sources.</p><p>The WSSP scripting uses several types of expressions:</p><ul><li>data element</li><li>array scanner</li><li>keyed element</li><li>indexed element</li><li>function call</li><li>boolean expression</li><li>string constant</li></ul><p>An alphanumeric string (such as <code>system</code> or <code>id</code>) is a data element name. The value of such an expression is the dataset value associated with this name. If the dataset does not have a specified key, the expression value is a null-value.<br> Example: the dataset contains the key <code>system</code> and its associated value is the <code>Sun Solaris</code> string, the value of the expression <code>system</code> is the string <code>Sun Solaris</code>.</p><p>The dataset dictionary is case-insensitive, so the data element names are case-insensitive, too.</p><p>An alphanumeric string followed by the <code>[]</code> symbols is interpreted as an index scanner name. It can be used only inside the <code>&lt;!--%%FOREACH name...--&gt;</code> ....<code>&lt;!--%%ENDFOR name--&gt;</code> structure where this index element is defined (see below). The index scanner names are case-insensitive.</p><p>An expression followed by the dot (<code>.</code>) symbol and an alphanumeric string is a keyed element. The expression before the dot symbol is calculated, and its value should be a dictionary. The alphanumeric string after the dot symbol specifies the key to be used to extract the value from that dictionary. If the value of the expression before the dot symbol is not a dictionary, or if it does not contain the specified key, the keyed element value is a null-value.<br> Keys can be specified as quoted strings, in this case they can specify non-alphanumeric symbols.<br> Example: the dataset contains the key <code>settings</code> and its associated value is the 2-element dictionary: <code>{OS = &quot;Sun Solaris&quot;; CPU = &quot;sparc&quot;;}</code>. The value of the <code>settings.OS</code> expression is the <code>Sun Solaris</code> string, the value of the <code>settings.&quot;OS1&quot;</code> expression is a null-value.</p><p>An expression followed by an <em>index expression</em> in square bracket symbols (<code>[index]</code>) is an indexed element. The expression before the square bracket is calculated, and its value should be an array or a dictionary. The <em>index expression</em> is calculated, and its value should be a number or a string representing a number. This number specifies which array element or dictionary key becomes the value of this indexed expression. If the value of the <em>index expression</em> is <code>0</code>, the first array element or the first dictionary key string is retrieved .<br> An <em>index expression</em> can be specified as a numeric constant.<br> If the value of the expression before the bracket symbol is not an array or a dictionary, or if the value of the <em>index expression</em> is not a number, or if the value of the <em>index expression</em> represents a number that is negative or is equal or greater than the number of array or dictionary elements, the value of the index expression is a null-value.</p><p>An alphanumeric string followed by the <code>(</code> symbol is a function call. Elements after the <code>(</code> symbol specify the function parameters, and they are followed by thes <code>)</code> symbol.<br> Function names are case-insensitive.<br> The list below specifies the available functions and their parameters.</p><p>Boolean expressions are two expressions separated with the <code>|</code> (OR) symbol, or with the <code>&amp;</code> (AND) symbol, or with the <code>^</code> (XOR) symbol.<br> You can use parentheses to enclose expressions:</p><p><code>expression1 | expression2</code><br><code>expression1 &amp; expression2</code><br><code>(expression1 &amp; expression2) | expression3</code></p><p>A boolean value of an expression is positive (true), if:</p><ul><li>the expression value is a string and it does not start with symbols <code>N</code>, <code>n</code>, <code>-</code>, or <code>0</code>, or</li><li>the expression value is non-zero number.</li><li>the expression value is a timestamp and it is not the &quot;remote past&quot; constant.</li></ul><p>A string constant is a sequence of symbols enclosed into quote symbols. The quote symbols and the backslash symbols must be prefixed (&quot;escaped&quot;) using the backslash <code>\\</code> symbol: <code>My \\&quot;test\\&quot; string</code>.</p><ul><li><p><code>SESSION(key)</code></p><p>This function can be used only in Session-based requests. The function value is the Session Dataset value associated with the string <em>key</em>. The <em>key</em> parameter can be specified as an alphanumeric string, or as a string constant.<br> Example: the <code>SESSION(accountName)</code> expression value is the name of the CommuniGate Pro Account this session is opened for.<br> The Session Dataset is case-insensitive. It contains the following keys and values:</p><table tabindex="0"><thead><tr><th>Key</th><th>Value</th></tr></thead><tbody><tr><td><code>ID</code></td><td>a string with the unique identifier of this session</td></tr><tr><td><code>accountName</code></td><td>a string with the session Account name</td></tr><tr><td><code>domainName</code></td><td>a string with the name of the Domain the session Account belongs to</td></tr><tr><td><code>filesRef</code></td><td>a string with the URL prefix needed to retrieve files from the session Skin</td></tr><tr><td><code>fullAccountName</code></td><td>a string with the session Account full name: <code>accountName@domainName</code></td></tr><tr><td><code>loginAddress</code></td><td>a string specifying the network (IP) address the user was using when initiating this session</td></tr><tr><td><code>loginTime</code></td><td>the timestamp with the session start time</td></tr><tr><td><code>selectableMailboxes</code></td><td>an array with the names of all &quot;selectable&quot; Mailboxes</td></tr><tr><td><code>addressBooks</code></td><td>an array with the names of all available Address Books</td></tr><tr><td><code>webFolders</code></td><td>an array with the File Storage folder names</td></tr><tr><td><code>selectedMailbox</code></td><td>a string with the name of the target Mailbox for the last Copy/Move operation</td></tr><tr><td><code>selectedAddressBook</code></td><td>a string with the name of the currently selected Address Book.</td></tr><tr><td><code>selectedWebFolder</code></td><td>a string with the name of the target File Storage folder for the last Store File In operation</td></tr><tr><td><code>webSiteEnabled</code></td><td>this <code>&quot;YES&quot;</code> string element exists if the storage limit for the File Storage is not set to zero</td></tr><tr><td><code>openMailboxes</code></td><td>a dictionary with all currently opened Mailboxes (each dictionary key is the Mailbox UTF8 name).</td></tr></tbody></table><p><a href="./WebApp.html#Session">CG/PL Web Applications</a> can insert additional elements into the Session Dataset.</p></li><li><p><code>SETTINGS(key)</code></p><p>This function can be used only in Session-based requests. The function value is the effective WebUser setting value associated with the string <em>key</em>. The <em>key</em> parameter can be specified as an alphanumeric string, or as a string constant.</p></li><li><p><code>ACCOUNTSETTINGS(key)</code></p><p>This function can be used only in Session-based requests. The function value is the effective Account setting value associated with the string <em>key</em>. The <em>key</em> parameter can be specified as an alphanumeric string, or as a quoted string.</p></li><li><p><code>INCLUDEARG(number)</code></p><p>This function can be used only inside an include file. The <em>key</em> should be a decimal number specifying the parameter number of the <code>&lt;!--%%INCLUDE--&gt;</code> element that invoked this include file.<br> The first parameter has the number 0.<br> If the <code>&lt;!--%%INCLUDE--&gt;</code> element did not specify enough parameters, the function value is a null-value.</p></li><li><p><code>EXISTS(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is not a null-value, or the string <code>&quot;NO&quot;</code> if the returned value is a null-value.</p></li><li><p><code>DOESNOTEXIST(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;NO&quot;</code> if the calculated value is not a null-value, or the string <code>&quot;YES&quot;</code> if the returned value is a null-value.</p></li><li><p><code>YESNO(expression)</code></p><p>The parameter is an expression. Its boolean value is calculated, and the value is positive (true), the function returns the string <code>&quot;YES&quot;</code>, otherwise it returns the string <code>&quot;NO&quot;</code>.</p></li><li><p><code>BOOLARRAY()</code></p><p>The value is a 2-element array, containing the strings <code>&quot;NO&quot;</code> and <code>&quot;YES&quot;</code>.</p></li><li><p><code>NOT(expression)</code></p><p>The parameter is an expression. Its boolean value is calculated, and the function returns a null-value if the value is positive (true), otherwise the function returns the string <code>&quot;YES&quot;</code>.</p></li><li><p><code>EQUALS(expression1 AND expression2)</code></p><p>Both expressions are calculated, and if the calculated values match (including the case when both expressions return a null-value), the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>EQUALSNOCASE(expression1 AND expression2)</code></p><p>Both expressions are calculated. The function returns the string <code>&quot;YES&quot;</code> if both calculated values are a null-value or if both values are strings and these strings match using the ASCII case-insensitive comparison operation. In all other cases the function returns a null-value.</p></li><li><p><code>EQUALS(expression AND string)</code></p><p>The value of the expression is calculated and compared with the string, specified as a quoted string. If the value matches the string, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>EQUALSNOCASE(expression AND string)</code></p><p>The value of the expression is calculated and compared with the string, specified as a quoted string. If the value matches the string using the ASCII case-insensitive comparison operation, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>ISINDEX(expression IN scanner)</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. The expression value is calculated, and if its numeric value matches the current index in the array this <em>scanner</em> is used for, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>ISFIRST(scanner)</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. If the current value of the array index is zero, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>ISLAST(scanner)</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. If the current value of the array index is equal to the number of array or dictionary elements minus one, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>ISEVEN(scanner)</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. If the current value of the index is even, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>ISHALF(scanner)</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. If the current value of the index is equal to the number of array or dictionary elements divided by 2, the function returns the string <code>&quot;YES&quot;</code>. Otherwise the function returns a null-value.</p></li><li><p><code>CHECKED(expression)</code></p><p>The parameter is an expression. Its boolean value is calculated, and if its positive (true), the function returns the string <code>&quot;checked&quot;</code>, otherwise the function returns a null-value.</p></li><li><p><code>REQUIRED(expression)</code></p><p>The parameter is an expression. Its boolean value is calculated, and if its positive (true), the function returns the string <code>&quot;required&quot;</code>, otherwise the function returns a null-value.</p></li><li><p><code>HASPARENTMAILBOX(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a string, representing a name of hierarchical Mailbox. Otherwise the function returns a null-value.</p></li><li><p><code>ISSTRING(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a string. Otherwise the function returns a null-value.</p></li><li><p><code>ISNUMBER(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a number. Otherwise the function returns a null-value.</p></li><li><p><code>ISARRAY(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is an array. Otherwise the function returns a null-value.</p></li><li><p><code>ISDICTIONARY(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a dictionary. Otherwise the function returns a null-value.</p></li><li><p><code>ISDATE(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a timestamp. Otherwise the function returns a null-value.</p></li><li><p><code>ISDATA(expression)</code></p><p>The parameter is an expression. Its value is calculated, and the function returns the string <code>&quot;YES&quot;</code> if the calculated value is a datablock object. Otherwise the function returns a null-value.</p></li><li><p><code>NULL()</code></p><p>The value of this function is a null-value.</p></li><li><p><code>EMPTYSTRING()</code></p><p>The value of this function is an empty string.</p></li><li><p><code>EMPTYARRAY()</code></p><p>The value of this function is an array with zero elements.</p></li><li><p><code>EMPTYDICTIONARY()</code></p><p>The value of this function is an empty dictionary.</p></li><li><p><code>CURRENTTIME()</code></p><p>This function value is a timestamp - the current global time.</p></li><li><p><code>SELECTEDLANGUAGE()</code></p><p>This function value is a string - the currently selected language name.</p></li><li><p><code>SELECTEDTIMEZONE()</code></p><p>This function value is a string - the currently selected time zone name.</p></li><li><p><code>STRING(key)</code></p><p>The value of this function is the object associated with the <em>key</em> in the Skin Text Dataset. This object should be a string, otherwise the function returns a null-value. The key can be specified either as a quoted string literal, or as an expression - the expression value is calculated and used as the key.</p></li><li><p><code>DICTIONARY(key)</code></p><p>The value of this function is the object associated with the <em>key</em> in the Skin Text Dataset. This object should be a dictionary, otherwise the function returns a null-value. The key can be specified either as a quoted string literal, or as an expression - the expression value is calculated and used as the key.</p></li><li><p><code>ARRAY(key)</code></p><p>The value of this function is the object associated with the <em>key</em> in the Skin Text Dataset. This object should be an array, otherwise the function returns a null-value. The key can be specified either as a quoted string literal, or as an expression - the expression value is calculated and used as the key.</p></li><li><p><code>TRANSLATE(string USING dictionary)</code></p><p>The <em>string</em> parameter is an expression that should return a string value; the <em>dictionary</em> parameter is an expression that should return a dictionary value. If the dictionary contains a string value for the key specified with the first parameter value, the function returns this string. Otherwise the value of the <em>string</em> parameter is returned;<br> Example: the dataset contains the element <code>boxName</code> with the string value <code>INBOX</code>, and the element <code>boxNames</code> with the dictionary value <code>{INBOX = Incoming; Trash = &quot;Trash Can&quot;;}</code>. The value of the <code>TRANSLATE(boxName USING boxNames)</code> expression is the <code>&quot;Incoming&quot;</code> string.</p></li><li><p><code>CONTAINS(string IN array)</code></p><p>The function returns the string <code>&quot;YES&quot;</code> if the value of the <em>array</em> parameter is an array, and the string is equal to one of the array elements. Otherwise the function returns a null-value.<br> The string can be specified either as a quoted string literal, or as an expression.</p></li><li><p><code>RANDOMELEMENT(array)</code></p><p>The <em>array</em> parameter is an expression that should return an array value; The value of this function is a randomly-selected element from that array.</p></li><li><p><code>MONTHNAMES()</code></p><p>The function returns a fixed array with 12 string elements: <code>(&quot;Jan&quot;,&quot;Feb&quot;,&quot;Mar&quot;,&quot;Apr&quot;,&quot;May&quot;,&quot;Jun&quot;,&quot;Jul&quot;,&quot;Aug&quot;,&quot;Sep&quot;,&quot;Oct&quot;,&quot;Nov&quot;,&quot;Dec&quot;)</code> .</p></li><li><p><code>WEEKDAYS()</code> and <code>WEEKDAYS(expression)</code></p><p>If expression is not specified, the function returns a fixed array with 7 string elements: (<code>Sun</code>,<code>Mon</code>,<code>Tue</code>,<code>Wed</code>,<code>Thu</code>,<code>Fri</code>,<code>Sat</code>).<br> If expression is specified, its result should be a string with one of the weekday names, and the function returns an array with 7 weekday names, starting with the specified weekday: <code>WEEKDAYS(startOfWeek)</code><br> returns <code>(&quot;Tue&quot;,&quot;Wed&quot;,&quot;Thu&quot;,&quot;Fri&quot;,&quot;Sat&quot;,&quot;Sun&quot;,&quot;Mon&quot;)</code> if the value of the startOfWeek variable is <code>&quot;Tue&quot;</code>.</p></li><li><p><code>KNOWNCHARSETS()</code></p><p>The function returns a fixed array with string elements - names of character sets known to the system <code>(&quot;ISO-8059-1&quot;,&quot;ISO-2022-jp&quot;,&quot;KOI8-R&quot;, ...)</code> .</p></li><li><p><code>KNOWNTIMEZONES()</code></p><p>The function returns a fixed array with string elements - names of time zones known to the system <code>(&quot;NorthAmerica/Pacific&quot;,&quot;Europe/Central&quot;,&quot;(+0900) Japan/Korea&quot;, ...)</code></p></li><li><p><code>REQUESTSECURE()</code></p><p>The function returns the string <code>&quot;YES&quot;</code> if the current HTTP request is secured (i.e. is sent via the HTTPS protocol), otherwise the function returns a null-value.</p></li><li><p><code>REQUESTRESOURCE()</code></p><p>The function returns the resource string (the local part of the URL with optional URL parameters) of the current HTTP request.</p></li><li><p><code>REQUESTSOURCEIP()</code></p><p>The function returns the string with the IP Network Address the HTTP request originated from.</p></li><li><p><code>SERVERVERSION()</code></p><p>The function returns the string with the current CommuniGate Pro Server version.</p></li></ul><h2 id="Text" tabindex="-1">Text Elements <a class="header-anchor" href="#Text" aria-label="Permalink to &quot;Text Elements {#Text}&quot;">​</a></h2><p>Text elements are specified using double percent markers. The body of a text element is an expression with an optional prefix.</p><ul><li><p><code>%%expression%%</code></p><p>The expression is calculated. If the expression value is a string or a number, it substitutes substitutes the text element in the resulting markup code.<br> Otherwise the entire text element is removed from the resulting markup code.</p></li><li><p><code>%%HTML:expression%%</code></p><p>The expression is calculated. If the expression value is a string or a number, it substitutes substitutes the text element using HTML escape symbols. A string value is converted from the UTF-8 charset into the required charset.<br> Otherwise the entire text element is removed from the resulting markup code.</p><ul><li><p>Example:</p><p>if the expression result is the <code>&quot;&gt;=GO=&gt;&quot;</code> string, the text element is substituted with:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&amp;gt;=GO=&amp;gt;</span></span></code></pre></div></li></ul></li><li><p><code>%%HTMLUTF8:expression%%</code></p><p>The expression is calculated. If the expression value is a string or a number, it substitutes substitutes the text element using HTML escape symbols (the string is not converted from the UTF-8 charset into the page charset).<br> Otherwise the entire text element is removed from the resulting markup code.</p><ul><li><p>Example:</p><p>if the expression result is the <code>&gt;=GO=&gt;</code> string, the text element is substituted with:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&amp;gt;=GO=&amp;gt;</span></span></code></pre></div></li></ul></li><li><p><code>%%URL:expression%%</code></p><p>The expression is calculated. If the expression value is a string or a number, it substitutes the text element using URL escape symbols.<br> Otherwise the entire text element is removed from the resulting markup code.</p><ul><li><p>Example:</p><p>if the expression result is the <code>&quot;Stop It?&quot;</code> string, the text element is substituted with:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>Stop%20It%3F</span></span></code></pre></div></li></ul></li><li><p><code>%%MAILBOXRAWNAME:expression%%</code></p><p>The expression is calculated. If the value is not a string, then the entire text element is removed from the resulting markup code. If the result is a string, it is converted from the IMAP-specified Mailbox name encoding into the UTF-8 charset, then it is converted into the required charset, and the converted string substitutes the text element using HTML escape symbols.</p></li><li><p><code>%%MAILBOXNAME:expression%%</code></p><p>The expression is calculated. If the value is not a string, then the entire text element is removed from the resulting markup code. If the result is a string X, the <code>TRANSLATE(X USING DICTIONARY(&quot;MailboxNames&quot;))</code> expression is calculated. Then the prefix works in the same way as the <code>MAILBOXRAWNAME:</code> prefix.</p></li><li><p><code>%%MAILBOXLASTNAME:expression%%</code></p><p>The expression is calculated. If the value is not a string, then the entire text element is removed from the resulting markup code. If the result is a string X, it is interpreted as a Mailbox name. If the Mailbox name is a hierarchical one, only the last part of the name is used, otherwise the entire name is used. The name (or its last part) is converted in the same way as for the <code>MAILBOXNAME:</code> prefix.</p></li><li><p><code>%%URLMAILBOXPARENT:expression%%</code></p><p>The expression is calculated. If the value is not a string, then the entire text element is removed from the resulting markup code. If the result is a string X, it is interpreted as a Mailbox name. If the Mailbox name is a hierarchical one, the name of the parent Mailbox is used. It is converted in the same way as for the <code>URL:</code> prefix. If the Mailbox name is not a hierarchical one, the element is removed from the resulting markup code.</p></li><li><p><code>%%JAVASCRIPT:expression%%</code></p><p>The expression is calculated. If the value is not a string or a number, then the entire text element is removed from the resulting markup code. If the result is a string or a number, it substitutes the text element using escape symbols needed for JavaScript strings. The result is converted into Unicode and all non-ASCII symbols are presented using the <code>\\uhhhh</code> escape sequences.</p><div class="tip custom-block"><p class="custom-block-title">Note</p><p>this prefix does not put any quotation marks around the string.</p></div><ul><li><p>Example:</p><p>if the expression result is the <code>&#39;What do &quot;they&quot; think?&#39;</code> string, the text element is substituted with:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>What do \\&quot;they\\&quot; think</span></span></code></pre></div></li></ul></li><li><p><code>%%JSON:expression%%</code></p><p>The expression is calculated. The expression is converted to a string using JSON (JavaScript Object Notation) representation, and this string substitutes the text element. Timestamp objects are represented as RFC822-formatted date strings.</p><ul><li><p>Example:</p><p>if the expression result is the <code>( &#39;What do &quot;they&quot; think?&#39;, #123)</code> array, the text element is substituted with:</p><div class="language-json vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">json</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">[</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;What do </span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">\\&quot;</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">they</span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">\\&quot;</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;"> think&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">, </span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">124</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">]</span></span></code></pre></div></li></ul></li><li><p><code>%%SIZE:expression%%</code></p><p>The expression is calculated. If the value is not a string or a number, then the entire text element is removed from the resulting markup code.<br> If the expression value is a string, it is converted into a numeric value (the number of bytes).<br> The string should contain some number and, optionally, the <code>k</code> or <code>K</code> suffix (in this case the number is multiplied by 1024), or the <code>m</code> or <code>M</code> suffix (in this case the number is multiplied by 1048576).<br> Alternatively, a string can start with a letter <code>u</code> or <code>U</code>, in this case the converted number of bytes is -1.<br> If the expression value is a number, its numeric value is used.<br> The resulting number is converted into a &quot;size string&quot;, using the dictionary retrieved with the <code>DICTIONARY(&quot;SizePictures&quot;)</code> expression. If the number is negative, the dictionary is used to translate the string <code>unlimited</code>, and the result is used to replace this text element using the same conversions as used for the a text element with the <code>HTML:</code> prefix.<br> The calculated number of bytes is checked to see if it represents an even number of Megabyte or Kilobytes, and that number is greater than one. Then a string value associated with the keys <code>&quot;M&quot;</code>, <code>&quot;K&quot;</code>, or <code>&quot;&quot;</code> (empty string) is retrieved from the dictionary. The string is expected to contain the <code>^0</code> symbol combination which is replaced with the number of megabytes, Kilobytes, or bytes specified with the <em>expression</em> value. The resulting string is processed with the method used for the <code>HTML:</code> text element prefix.<br> If the <code>DICTIONARY(&quot;SizePictures&quot;)</code> expression result is a null-value, or this dictionary does not contain a string value for the required key, the resulting string is built using the number and the key name (<code>20M</code>, <code>1345K</code>, <code>182345777</code>).</p></li><li><p><code>%%ROUNDSIZE:expression%%</code></p><p>This prefix works in the same way as the <code>SIZE:</code> prefix, but the numeric <em>expression</em> value can be modified: if the value is equal or larger than 10000, then it is converted into &quot;Kilo&quot; (value = value / 1024 * 1024), and if the value is equal or larger than 10240000, it is converted to &quot;Mega&quot; (value = value / 1048576 * 1048576).</p></li><li><p><code>%%TIME:expression%%</code></p><p>The expression is calculated.<br> If the result is a null-value, then the entire text element is removed from the resulting markup code.<br> If the result is a number, its value is interpreted as the number of seconds.<br> If the result is a string, it is converted into a numeric value (the number of seconds):<br> The string should contain some number and, optionally, the <code>s</code> or <code>S</code> suffix, or the <code>m</code> or <code>M</code> suffix (in this case the number is multiplied by 60), or the <code>h</code> or <code>H</code> suffix (in this case the number is multiplied by 3600), or the <code>d</code> or <code>D</code> suffix (in this case the number is multiplied by 86400).<br> The resulting number is converted into a &quot;time string&quot;, using the dictionary retrieved with the <code>DICTIONARY(&quot;TimePictures&quot;)</code> expression.<br> The calculated number of seconds is checked to see if it represents an even number of weeks, days, hours, or minutes, and if that number is greater than 1. Then a string value associated with the keys <code>weeks</code>, <code>days</code>, <code>hours</code>, <code>minutes</code>, or <code>seconds</code> is retrieved from the dictionary. The string is expected to contain the <code>^0</code> symbol combination which is replaced with the number of weeks, days, hours, minutes, or seconds specified with the <em>expression</em> value. The resulting string is converted from the UTF-8 into the required charset and the converted string substitutes the text element using HTML escape symbols.<br> If the <code>DICTIONARY(&quot;TimePictures&quot;)</code> expression result is a null-value, or this dictionary does not contain a string value for the required key, the resulting string is built using the number, a space, and the key name (<code>3 weeks</code>, <code>11 hours</code>, <code>5 seconds</code>).</p><ul><li><p>Example:</p><p>If the data element <code>elapsedTime</code> is the <code>2400</code> string, then the text element</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>%%TIME:elapsedTime%%</span></span></code></pre></div><p>will be substituted with the following string:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>40 minutes</span></span></code></pre></div><p>If the <code>DICTIONARY(&quot;TimePictures&quot;)</code> exists and contains the string <code>&quot;^0mins&quot;</code> as the <code>minutes</code> value, then the text element</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>%%TIME:elapsedTime%%</span></span></code></pre></div><p>will be substituted with the</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>40mins</span></span></code></pre></div><p>string.</p></li></ul></li><li><p><code>%%DATETIME(formatName):expression%%</code></p><p>The expression is calculated. If the value is not of the timestamp type, then the entire text element is removed from the resulting markup code. If the value is a timestamp, it is converted into a text string using the specified format.<br> The <em>format</em> parameter can be specified as an alphanumeric atom, or as a quoted string.<br> The format string is the result of the <code>DICTIONARY(&quot;DatePictures&quot;).formatName</code> expression. If this expression does not result in a string value, the <code>((^h:^m:^s ^W ^D-^M-^Y))</code> string is used instead.<br> The format string is processed by replacing the following symbol combinations with the actual timestamp value parts:</p><table tabindex="0"><thead><tr><th>symbols</th><th>substituted with</th></tr></thead><tbody><tr><td><code>^D</code></td><td>the day of month (2-digit)</td></tr><tr><td><code>^d</code></td><td>the day of month (1- or 2-digit)</td></tr><tr><td><code>^M</code></td><td>the month name (one of those returned with the MONTHNAMES() function), translated using <code>DICTIONARY(&quot;DatePictures&quot;)</code></td></tr><tr><td><code>^N</code></td><td>the month number (2-digit, from <code>01</code> to <code>12</code>)</td></tr><tr><td><code>^Y</code></td><td>the year number (2-digit)</td></tr><tr><td><code>^y</code></td><td>the year number (4-digit)</td></tr><tr><td><code>^s</code></td><td>the seconds value (2-digit)</td></tr><tr><td><code>^m</code></td><td>the minutes value (2-digit)</td></tr><tr><td><code>^H</code></td><td>the hours value (2-digit), from <code>00</code> to <code>23</code></td></tr><tr><td><code>^h</code></td><td>the hours value (1- or 2-digit), from <code>12</code>,<code>1</code> to <code>11</code></td></tr><tr><td><code>^t</code></td><td>the <code>AM</code> or <code>PM</code> string, translated using <code>DICTIONARY(&quot;DatePictures&quot;)</code></td></tr><tr><td><code>^w</code></td><td>the weekday number (Sun - 0)</td></tr><tr><td><code>^W</code></td><td>the weekday name (one of those returned with the WEEKDAYS() function), translated using <code>DICTIONARY(&quot;DatePictures&quot;)</code></td></tr></tbody></table><p>The resulting string is placed into the markup code using the <code>HTML:</code> prefix processing.</p></li><li><p><code>%%LOCALDATETIME(formatName):expression%%</code></p><p>Processing is the same as for the <code>DATETIME(formatName)</code> prefix, but the timestamp value is converted into the local time first.</p></li><li><p><code>%%DATETIMEAS(format):expression%%</code></p><p>The expression is calculated. If the value is not of the timestamp type, then the entire text element is removed from the resulting markup code. If the value is a timestamp, it is converted into a text string using the specified format.<br> The <em>format</em> parameter should be a quoted string or an expression with a string value. This string is used as the format string.</p></li><li><p><code>%%LOCALDATETIMEAS(format):expression%%</code></p><p>Processing is the same as for the <code>DATETIMEAS(format)</code> prefix, but the timestamp value is converted into the local time first.</p></li><li><p><code>%%DATE:expression%%</code></p><p>Processing is the same as for the <code>DATETIME(&quot;dateOnly&quot;)</code> prefix.</p></li><li><p><code>%%LOCALDATE:expression%%</code></p><p>Processing is the same as for the <code>LOCALDATETIME(&quot;dateOnly&quot;)</code> prefix.</p></li><li><p><code>%%DATEWEEK:expression%%</code></p><p>Processing is the same as for the <code>DATETIME(&quot;dayAndDate&quot;)</code> prefix.</p></li><li><p><code>%%DATETIME:expression%%</code></p><p>Processing is the same as for the <code>DATETIME(&quot;dateAndTime&quot;)</code> prefix.</p></li><li><p><code>%%LOCALDATETIME:expression%%</code></p><p>Processing is the same as for the <code>LOCALDATETIME(&quot;dateAndTime&quot;)</code> prefix.</p></li><li><p><code>%%DATETIMESHORT:expression%%</code></p><p>Processing is the same as for:</p><ul><li>the <code>DATETIME(&quot;timeOnly&quot;)</code> prefix if the timestamp value specified with the expression is &quot;very close&quot; to the current time (the absolute difference is less than 22 hours),</li><li>the <code>DATETIME(&quot;monthDate&quot;)</code> prefix if the timestamp value specified with the expression is &quot;close&quot; to the current date (the absolute difference is less than 180 days),</li><li>the <code>DATETIME(&quot;dateOnly&quot;)</code> prefix in all other cases.</li></ul></li><li><p><code>%%LOCALDATETIMESHORT:expression%%</code></p><p>Processing is the same as for the <code>DATETIMESHORT:</code> prefix, but the the expression value is converted into the local time first.</p></li><li><p><code>%%HTMLTRUNCATED(number):expression%%</code></p><p>The expression is calculated. If the value is not a string, then the entire text element is removed from the resulting markup code. If the result is a string, the string is truncated, then it is converted from the UTF-8 charset into the required charset, and the converted string substitutes the text element using HTML escape symbols. Not more than the <em>number</em> of symbols (&quot;glyphs&quot;) are substituted. If the string has more glyphs, the <code>..</code>symbols are added after the string.<br> Example: if the expression result is the <code>&quot;Test Subject&quot;</code> string, the text element substituted with the HTMLTRUNCATED(10) prefix is</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>Test Subje..</span></span></code></pre></div></li><li><p><code>%%HTMLTRUNCATED(expression1):expression%%</code></p><p>The same as the above, but the <em>expression1</em> parameter is an expression that is calculated. The <em>expression1</em> should have a numeric value. This value specifies the number of glyphs to be taken from the <em>expression</em> string.<br> Example: <code>HTMLTRUNCATED(STRING(&quot;FieldLength&quot;)):theField</code></p></li><li><p><code>%%HTMLSUBST(parameter0,parameter1,...):expression%%</code></p><p>The all parameterN expressions and the expression is calculated. If the value of the expression is not a string, then the entire text element is removed from the resulting markup code. If the result is a string, all its ^N substrings are replaced with the string values of parameterN expression (<code>^0</code> substrings are replaced with the value of parameter0, <code>^1</code> substrings are replaced with the value of parameter1, etc.). If the parameter value is not a string, the ^N substring is removed.<br> The resulting string is converted from the UTF-8 charset into the required charset, and the converted string substitutes the text element using HTML escape symbols.<br> Example: if the <code>text1</code> element of the Text Dataset is the <code>&quot;My String1&quot;</code> string, the var2 element of the Result Dataset is <code>My Var2</code>, and the <code>text2</code> element of the Text Dataset is the <code>&quot;comparing ^0 &amp; ^1&quot;</code> string, then the following code:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>%%HTMLSUBST(STRING(&quot;text1&quot;),var2):STRING(&quot;text2&quot;)%%</span></span></code></pre></div><p>will be substituted with</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>comparing My String1 &amp;amp; My Var2</span></span></code></pre></div><p>.</p></li><li><p><code>%%URLSUBST(parameter0,parameter1,...):expression%%</code></p><p>The same as the above, but the resulting string substitutes this text element using URL escape symbols.</p></li><li><p><code>%%INDEX:scanner%%</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. The scanner index decimal value substitutes this text element. For the first element the value 0 is used.</p></li><li><p><code>%%INDEX1:scanner%%</code></p><p>The <em>scanner</em> should be the name of the <code>&lt;!--%%FOREACH scanner IN ...--&gt;</code> construct surrounding the current portion of the script code. The scanner index decimal value substitutes this text element. For the first element the value 1 is used.</p></li><li><p><code>%%DUMP:expression%%</code></p><p>The expression is calculated. And the value <a href="./Data.html">textual representation</a> substitutes this text element.</p></li><li><p><code>%%LENGTH:expression%%</code></p><p>The expression is calculated.<br> If the value is a string, the string length (in bytes) substitutes this text element.<br> If the value is an array, the number of array elements substitutes this text element.<br> If the value is a dictionary, the number of dictionary key-value pairs substitutes this text element.<br> Otherwise, this text element is removed.</p></li><li><p><code>%%CENTS:expression%%</code></p><p>The expression is calculated. Its numeric value is converted into a numeric string with at least 3 decimal digits, and the period (<code>.</code>) symbol is inserted before the last 2 decimal digits. The resulting string substitutes this text element.</p></li></ul><h2 id="Structs" tabindex="-1">Structural Elements <a class="header-anchor" href="#Structs" aria-label="Permalink to &quot;Structural Elements {#Structs}&quot;">​</a></h2><p>The structural elements start with the <code>&lt;!--%%</code> symbols and end with the <code>--&gt;</code> symbols. The structural elements themselves are always removed from the resulting markup code.</p><ul><li><p><code>&lt;!--%%IF expression--&gt;</code></p><p>This structural element may be followed with one or more<br><code>&lt;!--%%ELIF expression--&gt;</code><br> element(s), then it may be followed with the<br><code>&lt;!--%%ELSE--&gt;</code><br> element, and then it must be followed with the<br><code>&lt;!--%%ENDIF--&gt;</code><br> element.</p><p>The boolean value of the expression in the IF element is calculated, then the values of the expressions in the ELIF elements (if any exists) are calculated, till one of those values is positive (true).<br> If an element with such a value is found, the portion of the script between this element and the following ELIF, ELSE, or ENDIF element is processed.<br> If an element is not found, and there is no ELSE element, the script between the IF and ENDIF elements is removed completely, otherwise the portion between the ELSE and ENDIF elements is processed.</p><ul><li><p>Example:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">&lt;!--%%IF EXISTS(lastLogin)--&gt;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">We have not seen you since &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">i</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;%%HTML:lastLogin%%&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">i</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">&lt;!--%%ELSE--&gt;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">Welcome, new user!</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">&lt;!--%%ENDIF--&gt;</span></span></code></pre></div><p>In this example, if the dataset contains the <code>lastLogin</code> element with the <code>20-Apr-2007</code> string value, this script portion will produce<br><code>We have not seen you since &lt;i&gt;20-Apr-2007&lt;/i&gt;</code></p><p>If the dataset does not contain the <code>lastLogin</code> element, this script portion will produce<br><code>Welcome, new user!</code></p></li></ul></li><li><p><code>&lt;!--%%FOREACH scanner in expression--&gt;</code><br> or<br><code>&lt;!--%%FORALL scanner in expression--&gt;</code></p><p>This structural element can be followed with the<br><code>&lt;!--%%EMPTYFOR scanner--&gt;</code><br> element, and must be followed with the<br><code>&lt;!--%%ENDFOR scanner--&gt;</code><br> element. All elements should have the same <em>scanner</em> alphanumeric string.</p><p>The value of the <em>expression</em> is calculated. The resulting value should be an array or a dictionary.<br> The portion of the script between the <code>&lt;!--%%FOREACH scanner</code> ...<code>--&gt;</code> and <code>&lt;!--%%EMPTYFOR scanner--&gt;</code> elements or (if the <code>&lt;!--%%EMTPYFOR scanner--&gt;</code> element does not exist) the portion of the script between the <code>&lt;!--%%FOREACH scanner</code> ...<code>--&gt;</code> and <code>&lt;!--%%ENDFOR scanner--&gt;</code> elements is processed repeatedly, for each array or dictionary element.</p><p>Expressions specified in that portion of the script (called the &quot;loop body&quot;) can use the <code>scanner[]</code> <em>scanner reference</em> to access the current element of the <em>expression</em> value.<br> If the <em>expression</em> is a dictionary, then the <code>scanner[*]</code> <em>key reference</em> can be used to access the current element key.<br> If the <em>expression</em> value is not an array or dictionary, or if it is an empty array or dictionary, and the <code>&lt;!--%%EMPTYFOR scanner--&gt;</code> element is specified, the portion of the script between the <code>&lt;!--%%EMPTYFOR scanner--&gt;</code> and <code>&lt;!--%%ENDFOR scanner--&gt;</code> elements is processed.</p><ul><li><p>Example:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">table</span><span style="--shiki-light:#B31D28;--shiki-light-font-style:italic;--shiki-dark:#FDAEB7;--shiki-dark-font-style:italic;"> border</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;File Name&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;File Size&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">  &lt;!--%%FOREACH elem in fileList--&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;%%HTML:elem[].name%%&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;%%elem[].size%%&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">  &lt;!--%%EMPTYFOR elem--&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> colspan</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">&amp;nbsp;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#6A737D;--shiki-dark:#6A737D;">  &lt;!--%%ENDFOR elem--&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">table</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div><p>In this example, the data element <code>fileList</code> is expected to be an array of dictionaries. Each dictionary is expected to contain string values for keys <code>name</code> and <code>size</code>.<br> If the <code>fileList</code> value is<br><code>({name=MyReport; size=2300;},{name=&quot;My Old Report&quot;; size=4000;})</code><br> then this portion of the script will produce the following HTML code:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">table</span><span style="--shiki-light:#B31D28;--shiki-light-font-style:italic;--shiki-dark:#FDAEB7;--shiki-dark-font-style:italic;"> border</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;File Name&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;File Size&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;MyReport&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;2300&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;My Old Report&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">    &lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;4000&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">td</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">  &lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">tr</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">table</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul><p>The <code>&lt;!--%%IF</code> ...<code>--&gt;</code> ...<code>&lt;!--%%ELSE--&gt;</code> ...<code>&lt;!--%%ENDIF--&gt;</code> constructs and the <code>&lt;!--%%FOREACH</code> ...<code>--&gt;</code> ... <code>&lt;!--%%ENDFOR</code> ...<code>--&gt;</code> constructs can be nested.</p></li><li><p><code>&lt;!--%%FOREACHINC scanner in expression--&gt;</code></p><p>The same as the <code>&lt;!--%%FOREACH</code> construct, but it repeats the &quot;loop body&quot; code portion one time more than the number of elements in the <em>expression</em> value array or dictionary.<br> For this additional run, the <code>scanner[*]</code> and <code>scanner[]</code> values are null-values.<br> If the <em>expression</em> value is not an array or a dictionary, the loop body is not repeated, and the portion between the <code>&lt;!--%%EMPTYFOR scanner--&gt;</code> and <code>&lt;!--%%ENDFOR scanner--&gt;</code> elements (if any) is processed.</p></li><li><p><code>&lt;!--%%FOREACHREV scanner in expression--&gt;</code></p><p>The same as the <code>&lt;!--%%FOREACH</code> construct, but it repeats the &quot;loop body&quot; code portion for each element in the <em>expression</em> value array or dictionary, taken in the reversed order (the last element first).</p></li><li><p><code>&lt;!--%%INCLUDE filename</code>[<code>( parameter1</code> [<code>, parameter2 ...</code> ]<code>)</code>]<code>--&gt;</code></p><p><em>filename</em> should be a quoted string or an expression with a string value.<br> The file with the <em>filename</em> name is retrieved from the same Skin this script is retrieved from. The file should contain some WSSP code. This code is executed within the current context (using the same dataset).<br> The resulting markup code is used to replace this structural element.</p><p>It is recommended to use the <code>.wssi</code> file name extension for files designed to be used with the INCLUDE element.</p><p>The INCLUDE elements can be nested, this means that <code>.wssi</code> files can include other <code>.wssi</code> files.</p><p>Unlike the <code>#include</code> operators in the C and C++ languages, this operator is a real operator, not a pre-processor operator. As a result, if an <code>&lt;!--%%INCLUDE filename--&gt;</code> element is used within a <code>&lt;!--%%FOREACH</code> ...<code>--&gt;</code> ... <code>&lt;!--%%ENDFOR</code> ...<code>--&gt;</code> construct, the <em>filename</em> code can be executed several times, once for each element of the array used in the FOREACH construct.</p><p>You can specify one or more parameters, enclosing them into parentheses and separating them with the comma symbol. Each parameter is an expression that is evaluated before the <em>filename</em> code is executed.<br> The <em>filename</em> code can reference its parameter values using the <code>INCLUDEARG</code> function.</p></li><li><p><code>&lt;!--%%NUMERICMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN (number1,number2,....,numberN)</code> [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element is substituted with a sequence of the <code>&lt;option value=&quot;value&quot;&gt;presentation</code> string elements.<br> &gt;The number of elements and the <em>value</em> used for each element are defined by the list of numeric numbers - <em>number1</em>,<em>number2</em>,....,<em>numberN</em>. These numbers should be specified in the ascending order, and these numbers should not be less than -1.<br> The <em>selected</em> expression is calculated, and its value should be a string. The string numeric value is used to add the keyword <code>selected</code> to the <code>&lt;option value=&quot;value&quot;&gt;</code> element that has the same <em>value</em>.<br> The <code>DISPLAY</code> keyword and the <em>dictionary</em> expression can be omitted. In this case, the <em>presentation</em> strings are the same as the <em>value</em> strings, this means that these strings are numbers.</p><ul><li><p>Example:</p><p>The dataset element <code>sizeLimit</code> is the <code>200</code> string.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%NUMERICMENU sizeLimit IN (-1,0,100,200,300)--&gt;</span></span></code></pre></div><p>will be substituted with the following markup text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;-1&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;0&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;100&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;100&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;200&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;200&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;300&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;300&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul><p>If the <code>DISPLAY</code> keyword and the <em>dictionary</em> expression are specified, the expression is calculated. If the expression value is a dictionary, then the <em>presentation</em> strings are the numeric values &quot;translated&quot; using this dictionary, the results are converted into the required charset and placed using the HTML escape symbols.</p><ul><li><p>Example:</p><p>The dataset element <code>sizeLimit</code> is the <code>200</code> string.<br> The Skin Text Dataset contains the <code>Limits</code> dictionary: <code>{&quot;-1&quot; = Unlimited; 0 = &quot;Off &amp; Shut&quot;;}</code>.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%NUMERICMENU sizeLimit IN (-1,0,100,200,300) DISPLAY DICTIONARY(&quot;Limits&quot;)--&gt;</span></span></code></pre></div><p>will be substituted with the following markup text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Unlimited&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Off </span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">&amp;amp;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> Shut&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;100&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;100&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;200&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;200&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;300&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;300&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul><p>If the keyword <code>DEFAULT</code> with the <em>selectedDefault</em> expression are specified, an additional <code>&lt;option value=&quot;-2&quot;&gt;defaultPresentation</code> string is added before the sequence. If the <em>selected</em> expression value is a null-value, this string element will have the keyword <code>selected</code> added.</p><p>The <em>defaultPresentation</em> string is the <code>DefaultValuePicture</code> string retrieved from the Skin Text Dataset. This string should contain the <code>^0</code> symbol combination. This symbol combination is substituted with the <em>selectedDefault</em> expression value. If the DISPLAY dictionary is specified, the <em>selectedDefault</em> expression value is translated first.</p><ul><li><p>Example:</p><p>The dataset element <code>sizeLimit</code> is the <code>200</code> string.<br> The dataset element <code>defLimit</code> is the <code>-1</code> string.<br> The Skin Text Dataset contains the <code>DefaultValuePicture</code> string: <code>default(^0)</code>.<br> The Skin Text Dataset contains the <code>Limits</code> dictionary: <code>{&quot;-1&quot; = Unlimited; 0 = &quot;Off &amp; Shut&quot;;}</code>.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%NUMERICMENU sizeLimit DEFAULT defLimit IN (-1,0,100,200,300) DISPLAY DICTIONARY(&quot;Limits&quot;)--&gt;</span></span></code></pre></div><p>will be substituted with the following markup (HTML) text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;default(Unlimited)&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Unlimited&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Off </span><span style="--shiki-light:#005CC5;--shiki-dark:#79B8FF;">&amp;amp;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;"> Shut&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;100&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;100&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;200&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;200&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;300&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;300&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul></li><li><p><code>&lt;!--%%TIMEMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN (time1,time2,....,timeN)</code> [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element is processed in the same way as the NUMERICMENU element. The <em>timeN</em> units specify time intervals converted to numeric values (the number of seconds) using the same algorithm as the algorithm used for the <a href="#TIMEPrefix"><code>TIME:</code></a> text elements.<br> The <em>selected</em> and <em>selectedDefault</em> expressions should return strings. Each string is converted into a numeric value (the number of seconds) using the same algorithm as the algorithm used for the <a href="#TIMEPrefix"><code>TIME:</code></a> text elements.<br> Values are translated using the specified <code>DISPLAY</code> dictionary. If the <code>DISPLAY</code> dictionary is not specified or it does not contain a string for the given value, the <em>presentation</em> time strings are composed using the same method as the method used for the <code>TIME:</code> text elements.</p></li><li><p><code>&lt;!--%%SIZEMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN (size1,size2,....,sizeN)</code> [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element is processed in the same way as the NUMERICMENU element. The <em>sizeN</em> units specify data sizes converted to numeric values (the number of bytes) using the same algorithm as the algorithm used for the <a href="#SIZEPrefix"><code>SIZE:</code></a> text elements.<br> The <em>selected</em> and <em>selectedDefault</em> expressions should return strings. Each string is converted into a numeric value (the number of bytes) using the same algorithm as the algorithm used for the <code>SIZE:</code> text elements.<br> Values are translated using the specified <code>DISPLAY</code> dictionary. If the <code>DISPLAY</code> dictionary is not specified or it does not contain a string for the given value, the <em>presentation</em> size strings are composed using the same method as the method used for the <code>SIZE:</code> text elements.</p></li><li><p><code>&lt;!--%%ENUMMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN valueSet</code> [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element is substituted with a sequence of the <code>&lt;option value=&quot;value&quot;&gt;presentation</code> string elements.</p><p>The number of elements and the <em>value</em> used for each element are defined by the value of the <em>valueSet</em> expression. This value should be an array of strings. The <em>value</em> in each element is the string index in the valueSet array result.</p><p>The <em>selected</em> expression is calculated, and its value should be a string. The keyword <code>selected</code> is added to the <code>&lt;option value=&quot;value&quot;&gt;</code> element created for the the <em>valueSet</em> array element that matches the <em>selected</em> expression result.</p><p>The <code>DISPLAY</code> keyword and the <em>dictionary</em> expression can be omitted. In this case, the <em>presentation</em> strings are the same as the <em>valueSet</em> result elements.</p><ul><li><p>Example:</p><p>The dataset element <code>color</code> is the <code>&quot;Green&quot;</code> string.<br> The dataset element <code>colors</code> is the <code>(&quot;Blue&quot;, &quot;Green&quot;, &quot;Red&quot;)</code> array.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%ENUMMENU color IN colors--&gt;</span></span></code></pre></div><p>will be substituted with the following markup (HTML) text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Blue&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;1&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Green&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Red&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul><p>If the <code>DISPLAY</code> keyword and the <em>dictionary</em> expression are specified, the expression is calculated. If the expression value is a dictionary, it is used to &quot;translate&quot; the <em>valueSet</em> array strings before converting them into the required charset and placing into the resulting markup text using the HTML escape symbols.</p><ul><li><p>Example:</p><p>The dataset element <code>color</code> is the <code>&quot;Green&quot;</code> string.<br> The dataset element <code>colors</code> is the <code>(&quot;Blue&quot;, &quot;Green&quot;, &quot;Red&quot;)</code> array.<br> The Skin Text Dataset contains the <code>Colors</code> dictionary: <code>{Blue = &quot;Night Blue&quot;; Green = &quot;Grass Green&quot;;}</code>.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%ENUMMENU color IN colors DISPLAY DICTIONARY(&quot;Colors&quot;)--&gt;</span></span></code></pre></div><p>will be substituted with the following markup (HTML) text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Night Blue&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;1&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Grass Green&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Red&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul><p>If the keyword <code>DEFAULT</code> with the <em>selectedDefault</em> expression are specified, an additional <code>&lt;option value=&quot;-1&quot;&gt;defaultPresentation</code> string is added before the sequence. If the <em>selected</em> expression value is a null-value, this string element will have the keyword <code>selected</code> added.</p><p>The <em>defaultPresentation</em> string is the <code>DefaultValuePicture</code> string retrieved from the Skin Text Dataset. This string should contain the <code>^0</code> symbol combination. This symbol combination is substituted with the <em>selectedDefault</em> expression value. If the <code>DISPLAY</code> dictionary is specified, the <em>selectedDefault</em> expression value is translated first.</p><ul><li><p>Example:</p><p>The dataset element <code>color</code> is the <code>&quot;Green&quot;</code> string.<br> The dataset element <code>defColor</code> is the <code>&quot;Blue&quot;</code> string.<br> The dataset element <code>colors</code> is the <code>(&quot;Blue&quot;, &quot;Green&quot;, &quot;Red&quot;)</code> array.<br> The Skin Text Dataset contains the <code>DefaultValuePicture</code> string: <code>default(^0)</code>.<br> The Skin Text Dataset contains the <code>Colors</code> dictionary: <code>{Blue = &quot;Night Blue&quot;; Green = &quot;Grass Green&quot;;}</code>.<br> The element:</p><div class="language- vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang"></span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span>&lt;!--%%ENUMMENU color DEFAULT defColor IN colors DISPLAY DICTIONARY(&quot;Colors&quot;)--&gt;</span></span></code></pre></div><p>will be substituted with the following HTML text:</p><div class="language-html vp-adaptive-theme"><button title="Copy Code" class="copy"></button><span class="lang">html</span><pre class="shiki shiki-themes github-light github-dark vp-code" tabindex="0"><code><span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;-1&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;default(Night Blue)&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;0&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Night Blue&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;1&quot;</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> selected</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Grass Green&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span>
<span class="line"><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&lt;</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#6F42C1;--shiki-dark:#B392F0;"> value</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">=</span><span style="--shiki-light:#032F62;--shiki-dark:#9ECBFF;">&quot;2&quot;</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;Red&lt;/</span><span style="--shiki-light:#22863A;--shiki-dark:#85E89D;">option</span><span style="--shiki-light:#24292E;--shiki-dark:#E1E4E8;">&gt;</span></span></code></pre></div></li></ul></li><li><p><code>&lt;!--%%BOOLMENU selected</code> [<code>DEFAULT selectedDefault</code> ] [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element is substituted with a sequence of the <code>&lt;option value=&quot;value&quot;&gt;presentation</code> string elements in the same way the ENUMMENU element is processed.<br> Unlike the ENUMMENU element, this element does not contain the <code>IN valueSet</code> part: the built-in array <code>(&quot;NO&quot;,&quot;YES&quot;)</code> array is used instead.</p></li><li><p><code>&lt;!--%%MAILBOXMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN mailboxList</code> [<code>NOINBOX</code> ]<code>--&gt;</code></p><p>This element is substituted with a sequence of the <code>&lt;option value=&quot;value&quot;&gt;presentation</code> string elements in the same way the ENUMMENU element is processed.<br> The values of the <em>selected</em> and <em>selectedDefault</em> expressions are converted in the same way as they are converted for a text element with the <code>MAILBOXNAME:</code> prefix, using the <code>MailboxNames</code> dictionary from the Skin Text Dataset, this is why MAILBOXMENU elements do not have the DISPLAY part.<br> If the NOINBOX keyword is specified, the INBOX Mailbox (if exists in <em>mailboxList</em>) is not displayed.</p></li><li><p><code>&lt;!--%%DAYTIMEMENU selected</code> [<code>DEFAULT selectedDefault</code> ] [<code>PERIOD timePeriod</code> ] <code>--&gt;</code></p><p>This element is substituted with a sequence of the <code>&lt;option value=&quot;value&quot;&gt;presentation</code> string elements to form a time-of-day menu. Each value is the time in seconds, and presentation is a string presenting this time using the &quot;hourMinute&quot; format (or the &quot;^H:^m&quot; format string, if the &quot;hourMinute&quot; element is not found in the <code>DICTIONARY(&quot;DatePictures&quot;)</code> data.<br> The time values are generated from the midnight (<code>00:00</code>) till 23:59 with the timePeriod step (specified in minutes).<br> If the PERIOD keyword and the timePeriod value are not specified, the 30 minutes (1800 seconds) step is used.<br> If the PERIOD keyword is specified, the timePeriod value can be either a numeric constant specifying the period in minutes, or an expression. The expression value is calculated and converted to a number. This number specifies the period in seconds.<br> The <em>selected</em> and <em>selectedDefault</em> expressions should have numeric values - the currently selected (and the default) time-of-day (seconds from midnight).</p></li><li><p><code>&lt;!--%%CALENDARTIMEMENU selected</code> [<code>PERIOD timePeriod</code> ] <code>--&gt;</code></p><p>This element creates the same HTML menu as the DAYTIMEMENU element.<br> The <em>selected</em> expression value should be a timestamp. The time part of that value is used as the currently selected time-of-day value.</p></li><li><p><code>&lt;!--%%LOCALCALENDARTIMEMENU selected</code> [<code>PERIOD timePeriod</code> ] <code>--&gt;</code></p><p>This element creates the same HTML menu as the DAYTIMEMENU element.<br> The <em>selected</em> expression value should be a timestamp value. The value is converted to the local time zone, and time part of the converted value is used as the currently selected time-of-day value.</p></li><li><p><code>&lt;!--%%STRINGMENU selected</code> [<code>DEFAULT selectedDefault</code> ] <code>IN valueSet</code> [ <code>DISPLAY dictionary</code>]<code>--&gt;</code></p><p>This element works in the same way as the ENUMMENU element, but the values used are the actual data values, not their numbers as in the ENUMMENU element. The default value has the string value <code>default</code>.</p></li><li><p><code>&lt;!--%%CALENDARDATECONTROL selected NAME name</code> [<code>DAYSBEFORE before</code>] [<code>DAYSAFTER after</code>] [<code>CANNEVER</code> ] <code>--&gt;</code></p><p>This element composes a control or set of controls for a calendar date specified using the <em>selected</em> expression. The expression value should be a timestamp, otherwise the whole element is removed from the resulting markup code.</p><p>If the specified time is more than <em>before</em> days earlier than the current date or if it is more than <em>after</em> days later than the current date, then this element is substituted with a text with several text controls. Otherwise a menu control is composed.</p><p>The <em>before</em> and <em>after</em> values should be specified as numbers.<br> If the <em>before</em> value is not explicitly specified, it is set to 7.<br> If the <em>after</em> value is not explicitly specified, it is set to 31.<br> If the CANNEVER keyword is used, the control menu displays the <code>Never</code> item with the <em>remote future</em> value. If the selected value is equal to the <em>remote future</em> value, a menu control is composed.</p><p>The <em>name</em> element should be a string.</p><p>To compose text controls, the <code>dateOnly</code> format string is taken from the <code>DatePictures</code> dictionary, and the text control codes are used to substitute its <code>^D</code>, <code>^M</code>, <code>^Y</code>, and <code>^y</code> symbol combinations. Each control has the specified <em>name</em> with a prefix: the day control has the <code>name-D</code> name, the month control has the <code>name-M</code> name, the 2-digit year control has the <code>name-Y</code> name, and the 4-digit year control has the <code>name-y</code> name.</p><p>If a menu is to be composed the <code>select</code> markup language element is generated. It has the specified <em>name</em> name. The menu contains an element for the current date, <em>before</em> elements for the previous dates, and <em>after</em> elements for the future dates. The elements are formatted using the <code>dayAndMonthDate</code> format string from the <code>DatePictures</code> dictionary.<br> An additional element with the <code>...</code> text is added for the &quot;after+1&quot; date. When this element is selected, the selected date value moves outside the &quot;menu-covered&quot; range, allowing the user to use text controls and select an arbitrary date.</p></li><li><p><code>&lt;!--%%LOCALCALENDARDATECONTROL selected NAME name</code> [<code>DAYSBEFORE before</code>] [<code>DAYSAFTER after</code>] <code>--&gt;</code></p><p>This element creates the same markup code as the CALENDARDATECONTROL element.<br> The <em>selected</em> expression value should be a timestamp. The value is assumed to be expressed in the global (GMT) terms, so the value is converted to the local time first.</p></li></ul>`,33)]))}const k=t(n,[["render",o]]);export{u as __pageData,k as default};
